#!/usr/bin/perl 

###################################################################
# Config.pl : Configuration script for InterProScan v4.2
#
# Developed by Emmanuel Quevillon  - tuco@ebi.ac.uk
# Rewritten by Sarah Hunter - hunter@ebi.ac.uk
#
# Please email the InterProScan mailing list for comments or questions
#              interhelp@ebi.ac.uk
#
# Copyright EMBL/EBI (http://www.ebi.ac.uk)
#
# You may distribute this module under the same terms as perl itself
#
# $Id: Config.pl,v 1.6 2005/11/16 16:26:15 hunter Exp $
####################################################################

use strict;
use English;

my  $PWD = `pwd`;
chomp($PWD);
my $PERL_CMD = `which perl`;
chomp($PERL_CMD);


my @modules = qw(XML::Parser XML::Quote DB_File Fcntl English File::Basename File::Copy File::Path File::Spec::Functions Sys::Hostname Mail::Send FileHandle IO::Scalar CGI URI::Escape IO::String);
my @homechange = qw(/bin/wget.pl /bin/ResubmitJobs.pl /bin/iprscan /bin/converter.pl /bin/iprscan_wrapper.pl /bin/ProDomBlast3i.pl /bin/index_data.pl /bin/executor.pl /bin/executorfp.pl /bin/pantherScore.pl /conf/sixpack.sh /conf/seqret.sh /bin/out2raw.pl /bin/meter.pl);
#Supported queueing systems
my %queueS = ('lsf42' => 'bsub', 'pbs54' => 'qsub',  'sge6' => 'qsub');

my ($Tip, $res, $insDone, $setcnk, $make, $set_path, $install_path) = ("", "", "", "", "", "", "");
my ($perl_path, $applset, $installed_arch, $res_server, $suggest_path) = ("", "", "", "", "");
my ($UseQ, $Qsys, $local_server, $user, $local_arch, $serv, $user, $usetaxo) = ("", "", "", "", "", "", "", "");
my ($globalCluster, $globalQueue) = (undef, undef);
my ($HOME) = ("");

print STDOUT "\n####################################################################################################\n";
print STDOUT "# \t\t Welcome to the InterProScan v4.2 installation script.\n#\n";
print STDOUT "#  Tips:\n";
print STDOUT "#     - If you are happy with the suggestion between [], just press <enter>\n";
print STDOUT "#     - Whatever is shown between '/' and '/' is considered a valid input pattern\n";
print STDOUT "####################################################################################################\n\n\n";

$insDone = (-e "$PWD/.installed" ? 'n' : 'y');

if(-f "$PWD/.installed"){
    unless(open(INS, "<$PWD/.installed")){
	die "FATAL: Could not open $PWD/.installed : $!\n";
    }
    while(<INS>){
	chomp;
	if($ARG =~ /^\s*PERL_PATH=(\S+)/){
	    $PERL_CMD = $1;
	    $PERL_CMD =~ s/[;\']+//g;
	}
	if($ARG =~ /^\s*IPRSCAN_HOME=(\S+)/){
	    $HOME=$1;
	    $HOME =~ s/[;\']+//g;
	}
    }
    close(INS);
}else{
    unless(open(INS, ">$PWD/.installed")){
	die "FATAL: Could not open $PWD/.installed : $!\n";
    }
    print INS "PERL_PATH=$PERL_CMD\n";
    print INS "IPRSCAN_HOME=$PWD\n";
    close INS;
}

$insDone =  get_user_prompt('Reconfigure everything? (first time install)', $insDone, '(y|n)');

$Tip = "!\n! InterProScan needs to know where it is installed and where perl is installed\n!";

undef($set_path);

while(!defined($set_path)){
    $set_path = get_user_prompt('Do you want to set paths to perl and the installation directory?', $insDone, 'y|n', $Tip);
    unless( defined($set_path) && ( $set_path =~ /^\s*[yn]\s*$/ ) )
      {
	  print STDOUT "WARNING: The input is not valid (it should be 'y' or 'n'), try again.\n";
	  undef($set_path);
      }
}

#Setting paths...
if($set_path eq "y"){
    
    ## Verifying the installation path
    undef($install_path);
    $Tip = "!\n! If your servers are using a shared file system, such as NFS, InterProScan is able to utilise this\n"
        . "! and perform distributed computing across multiple servers\n"
        . "! You will need set up the full UNIX path to the installation directory.\n"
        . "! You also need to ensure that the InterProScan installation is on a SHARED DISK.\n!";
    
    while ( !defined($install_path) )
      {
	  $suggest_path = $PWD;
	  $suggest_path = $HOME if($HOME ne "");
	  chomp($suggest_path);
	  
	  $install_path = get_user_prompt('Please enter the full path for the InterProScan installation', $suggest_path, '.+', $Tip);
	  
	  unless ( -d $install_path )
	    {
		print STDOUT "WARNING: The input is not valid (it is not a directory), try again.\n";
		undef($install_path);
	    }
      }
    chomp($install_path);
    patch_file("$PWD/.installed", "IPRSCAN_HOME", "$install_path", 1);
    $HOME = $install_path;
    
    #Setting the iprscan home installation path
    for(@homechange){
	my $file = $install_path . $ARG;
	patch_file($file, "\\\$ENV{IPRSCAN_HOME}", "$install_path");
	patch_file($file, "IPRSCAN_HOME", "$install_path", 1);
    }
    
    $Tip = "!\n! You can specify a particular installation of Perl, besides the default, for use in InterProScan\n"
	."! Please note that InterProScan needs Perl5 or later (available from http://www.cpan.org/)\n"
	."! Various additional perl modules are also required.  These are listed in the installation notes\n"
	."! so please check that they are present\n!";

    undef($res);
    
    while(!defined($res)){
	$res = get_user_prompt("Do you want to set another Perl command in place of [$PERL_CMD]?", 'n' , 'y|n');
	unless(defined($res)){
	    print STDOUT "WARNING: The input is not valid must be \'y\' or \'n\', try again\n";
	    undef($res);
	}
    }
    if($res eq 'y'){
	undef($res);
	$suggest_path = $PERL_CMD;
	$res = which_binaries('Perl5', $suggest_path, '-v', 'v(ersion *)?(5|6)', $Tip);
	$PERL_CMD = $res;
	chomp($PERL_CMD);
	patch_file("$PWD/.installed", "PERL_PATH", "$PERL_CMD", 1);
    }
    #Changing the Perl path in all .pl scripts
    print STDOUT ">>Changing Perl path in scripts ... ";
    my @Perlfiles = `ls $PWD/bin/*.pl`;
    push(@Perlfiles, "$PWD/bin/iprscan");
    chomp(@Perlfiles);
    foreach my $file (@Perlfiles)
      {
	  unless(-f $file){
	      print STDOUT "\nWARNING: $file does not exist : $!. This may cause problems with your installation\n\n";
	      exit;
	  }
	  my $nb_lines = `wc -l $file`;
	  chomp($nb_lines);
	  $nb_lines =~ s/\s*(\d+).*$/$1/;
	  $nb_lines --;
	  `( echo "#!$PERL_CMD" ; tail -$nb_lines $file ) > $file~ ; mv $file~ $file ; chmod +x $file`;
      }
    print STDOUT ">>DONE\n";
    
    
    print STDOUT "\n>>Checking if your Perl installation has the modules required by InterProScan...\n";
    my $nomod=0;
    
    for(@modules){
	unless(open(MOD, ">$PWD/test.pl")){
	    die "FATAL: Could not open $PWD/test.pl in writing mode to test for presence of modules :$!\n";
	}
	print MOD "#!$PERL_CMD\n";
	print MOD "use $ARG;\n";
	print MOD "1;\n";
	close MOD;
	chmod(0777, "$PWD/test.pl");
	system("./test.pl");
	if($CHILD_ERROR){
	    print STDOUT "WARNING: Module $ARG is not installed\n\n";
	    $nomod++;
	    unlink("$PWD/test.pl");
	    #exit(1);
	}
    }
    unlink("$PWD/test.pl");
    if ($nomod) {
	print STDOUT "FATAL: $nomod modules are missing, please install them or check your LD_LIBRARY_PATH and PATH\n";
	print STDOUT "environmental variables, then relaunch Config.pl.  Sorry for any inconvenience\n\n";
	exit(1);
    }

    print STDOUT ">>All modules needed are already installed.\n\n";
    
} #end if($set_path eq 'y')

##  Set chunk size
$setcnk = $insDone;
$Tip = "!\n! To cope with bulk jobs and to enable efficient parallelization, InterProScan splits input files\n"
	."! with FASTA formatted sequences into smaller parts (chunks). Default size is \"10\" sequences per chunk\n!";

$setcnk = get_user_prompt('Do you want to setup chunk size', $insDone, '(y|n)', $Tip);
my $chunkz;

if ($setcnk eq "y")
  {
      $Tip = "!\n! Here you should specify the maximum number of sequences allowed in each part (chunk).  Please note\n"
           ."! it is not recommended to have more then 3000 chunks.\n!";

      $chunkz = get_user_prompt('Enter chunk size', '10', '[0-9]+', $Tip);
  }
print STDOUT "\n";


#Setup maximum length, maximum number of aa and nt seqeunces.

$Tip = "!\n! You can configure InterProScan to limit the number of input sequences (protein & nucleotide),\n"
  . "! set a maximum length for nucleotide sequences and a minimum length for protein sequences.\n"
  . "! You can also configure the minimum length for a translated orf and use the codon table value\n"
  . "! to translate the dna/rna sequence into six frames (http://www.ebi.ac.uk/cgi-bin/mutations/trtables.cgi).\n!";

my($maxinaa, $maxinnt, $maxlennt, $minlenaa, $minorflen, $cdntable) = (undef, undef, undef, undef, undef, undef);

my $setseq = get_user_prompt('Do you want to configure this?', $insDone, '(y|n)', $Tip);

if($setseq eq 'y'){
    $maxinaa   = '1000';
    $maxinnt   = '100';
    $maxlennt  = '10000';
    $minlenaa  = '5';
    $minorflen = '50';
    $cdntable  = '0';

    $maxinaa = get_user_prompt('Enter the maximum number of input protein sequences allowed', $maxinaa, '\d+');
    $maxinnt = get_user_prompt('Enter the maximum number of input nucleic sequences allowed', $maxinnt, '\d+');
    $maxlennt = get_user_prompt('Enter the maximum length (in nucleic acids) for a nucleotide sequence', $maxlennt, '\d+');
    $minlenaa = get_user_prompt('Enter the minimum length (in amino acids) for a protein sequence', $minlenaa, '\d+');
    $minorflen = get_user_prompt('Enter the minimum allowed length of a translated orf', $minorflen, '\d+');
    $cdntable = get_user_prompt('Enter the default codon table value to use to translate dna/rna in six frames', $cdntable, '\d+');
}
print STDOUT "\n";


$Tip = "!\n! InterProScan currently encorporates searching against 13 databases. You have the choice whether or not to\n"
	."! set up a particular search tool against a database and how exactly that application will be configured to run\n"
	."! in this next section. You will configure the queue systems (if any) you will use, first\n!";

$applset = get_user_prompt('Do you want to setup applications (if you don\'t, no applications will be included in InterProScan by default)?', $insDone, '(y|n)', $Tip);
my @setAppl = ();

if($applset eq "y"){
    
    $Tip = "!\n! InterProScan is able to execute the applications via a UNIX QUEUING system which has an INTERACTIVE mode (such as LSF).\n"
	."! Currently, we mainly support LSF (which is used at EBI) however, we also provide configuration files for other systems\n"
	."! which are listed in the documentation\n!";

    $UseQ = get_user_prompt('Do you wish to use a queue system?', 'n', '(y|n)', $Tip);
    $Qsys = undef;
    
    my $cluster = 'foo';
    my $queuename = 'blahblah';
    my $globans = undef;
    
    if($UseQ eq "y"){

	print STDOUT "!\n! VERY IMPORTANT NOTICE: \n",
		"! Applications from InterProScan will use a specific directory (iprscan/bin/binaries) to find application binaries\n",
		"! and execute them.  'binaries' is a symbolic link pointing to a directory with a name that corresponds to the\n",
		"! OS architecture (OSF1|Linux|AIX|SunOS|Darwin|IRIX).  If you have different architectures on any of the nodes\n",
		"! used by your queueing system, you must make sure that the link they see points to the appropriate architecture\n",
		"! directory.\n",
		"! (e.g.) Host A is Linux, Host B is OSF1.  Be sure that, on host A, the binaries link points to a directory called\n",
		"!       'Linux' and on host B, to 'OSF1'\n!\n",
		"\nPlease choose one queueing system from the list. If you don't want to use this queueing system, just type '<enter>'\n\n";

	sleep(1);

      QUEUE:

	for(keys %queueS){
	    $Qsys = get_user_prompt("Do you want to use $ARG?", 'n', '(y|n)');
	    if($Qsys eq 'y'){
		my $cmd = $queueS{$ARG};
		my $err = "$HOME/cmd.errors";
		my $res = `which $cmd 2>$err`;
		my $queue = $ARG;
		
		if(-s $err>0){
		    $res = `cat $err`;
		    $res =~ s/\n//g;
		    
		    sleep(1);
		    print STDOUT "ERROR : It seems that $cmd is not found or perhaps not in your path. Please fix this first, then relaunch Config.pl\n";
		    print STDOUT "If you want to use the $ARG queuing system, you need to install InterProScan on a host which can run $ARG, or as a user\n";
		    print STDOUT "who has the $cmd command in her/his path and is able to run it.\n\n";
		    print STDOUT "\n$res\n\n";
		    unlink($err);
		    exit;
		}
		unlink($err);
		$Qsys = $queue;
		chomp($Qsys);
		$UseQ = 'y';
		last;
		#We configure conf file for each applications later, when asking user which application he wants to run.
	    }else{
		$Qsys = undef;
		$UseQ = 'n';
	    }
	}

	unless(defined($Qsys)) { 
	    $Tip = "!\n! You did not choose any queueing system. Do you want Config.pl to show you the list of supported queueing systems again? "
		. "! If you don't choose a queue system, 'local' is set by default\n!";

	    my $showagain = get_user_prompt("", 'y', '(y|n)', $Tip);

	    unless ($showagain eq 'n'){
		goto QUEUE;
	    }else{
		print STDOUT "\n>>By default 'local' has been set for all your applications. If you want to set a queueing system\nto be run with your applications, relaunch Config.pl.\n";
		goto LOCAL;
	    }
	}
	
	#Set GLOBAL values for cluster name and queue name?
	
      GLOBAL:
	
	print STDOUT "!\n! You can now configure global settings for your queue system for all the applications (such as a ",
			"! global cluster name and/or a global queue name)\n!\n";

	$globans = get_user_prompt("Do you want to set a global cluster name for all applications?", 'n', '(y|n)');
	chomp($globans);

	if($globans eq 'y'){
		$globalCluster = get_user_prompt("Please enter the global cluster name you want to use for all applications", '', '([^\s\;\,\:]+)');
		chomp($globalCluster);
		$globans = 'n';
	}

	$globans = get_user_prompt("Do you want to set a global queue name for all applications?", 'n', '(y|n)');
	chomp($globans);

	if($globans eq 'y'){
		$globalQueue = get_user_prompt("Please enter the name of the global queue you want to use for all applications", '', '([^\s\;\,\:]+)');
		chomp($globalQueue);
		$globans = 'n';
	}

	print STDOUT ">>You will run InterProScan with $Qsys queueing system using:\n";
	if(defined($globalQueue)){
		print STDOUT ">>\t-Global cluster name '$globalCluster'.\n";
	}
	if(defined($globalQueue)){
		print STDOUT ">>\t-Global queue name '$globalQueue'.\n";
	}

	$globans = get_user_prompt("\nIs all this information correct?", 'n', 'y|n');
	chomp($globans);
	if($globans ne 'y'){
		goto GLOBAL;
	}
	undef($globans);
    }

  LOCAL:

    my @applications;
    my $appldesc = { };
    my $applConf = { }; #queue name for each application, if user wants to have different queue 

    print STDOUT ">>Checking your local architecture for InterProScan compatibility\n";

    #Read supported applications
    unless(open(APPL, "<$PWD/conf/iprscan.conf")){
	die "FATAL: Could not open $PWD/conf/iprscan.conf : $!";
    }
    while(<APPL>){
	chomp;
	if($ARG =~ /^\s*defaults.applications=(\S+)/){
	    @applications = split(/\,/, $1);
	}
	if($ARG =~ /^\s*appl\.desc\.(\S+)=(.+)/){
	    my $v = $2;
	    chomp($v);
	    $appldesc->{$1} = $v;
	}
    }
    close(APPL);

    #Defining local host name and arch
    my $supported_arch = 'OSF1|IRIX64|Linux|SunOS|Darwin|AIX';
    ($local_server = `hostname`);
    ($local_arch = `uname`);
    ($serv = $local_server);
    $local_server =~ s/\n//g;
    $local_arch =~ s/\n//g;
    $serv =~ s/\n//g;

    #Check first if the local arch is able to run InterProScan
    unless($local_arch =~ /$supported_arch/){
	print STDOUT "FATAL: Sorry, your architecture $local_arch is not supported\n";
	print STDOUT "Supported architectures are : $supported_arch\n";
	exit;
    }
    
    #Ask user to use applications or not
    my $seen = 0;

    for(@applications){
	
	my $rep = ($ARG eq "tmhmm" || $ARG eq "signalp" ? 'n' : 'y');
	$Tip = "! ".ucfirst($ARG) . " " . $appldesc->{$ARG};
	my $useappl = get_user_prompt("Do you want to use $ARG ?", $rep, '(y|n)', $Tip);
	my $linkdone = "$HOME/bin/binaries";
	my $arch = `uname`;
	$arch =~ s/\n//g;

	if($useappl eq 'y'){
	    $applConf->{$ARG}->{run} = 1;
	    push(@setAppl, $ARG);
	    
	    if($UseQ eq 'y'){
		$applConf->{$ARG}->{usequeue} = 1;
		my $UseResource;

		unless(-l $linkdone){
		    unless(-d "$HOME/bin/$arch"){
			print STDOUT "WARNING: The \"$arch\" UNIX OS IS supported but the binaries directory $arch is NOT installed\n";
			print STDOUT "Please install it under the same path as your InterProScan installation: $HOME/bin/$arch\n";
			print STDOUT "Then create a symbolic link called binaries to this $arch directory\n"; 
			print STDOUT "\te.g.: cd $HOME/bin; ln -s $arch binaries\n";
			print STDOUT "Check the FTP site, (ftp.ebi.ac.uk/pub/databases/interpro/iprscan/BIN) for the binaries tarball";
			print STDOUT "or try another host. You will have to do it for each server having different architecture than the\n";
			print STDOUT "install host ($local_server).\n\n";
		    }else{
			my $res = `cd $HOME/bin; ln -s $arch binaries`;
			if($CHILD_ERROR){
			    print STDOUT "\tERROR: Unable to create symbolic link 'ln -s $arch binaries': $!\n";
			    print STDOUT "\tPlease correct the error or create the symbolic link manually if necessary\n\n";
			}
		    }
		} #end else

		$Qsys = get_user_prompt("Which queueing system do you want to use?", "$Qsys", "lsf42|pbs54|sge|condor") unless(defined($Qsys));
		chomp($Qsys);
		$applConf->{$ARG}->{queue} = $Qsys if(defined($Qsys));

		unless(defined($globalCluster)){
		    $Tip = "!\n! You can specify a cluster name (resource) where to run $ARG\n!";
		    $UseResource = get_user_prompt("Do you want to launch $ARG on a specific cluster?", 'n', 'y|n', $Tip);
		    chomp($UseResource);
		    
		    #Use specific resource?
		    if($UseResource eq 'y'){
			$UseResource = get_user_prompt('Enter cluster name', $cluster, '\S+');
			chomp($UseResource);
			$applConf->{$ARG}->{resource} = $UseResource;
			$cluster = $UseResource;
		    }
		}else{
		    $applConf->{$ARG}->{resource} = $globalCluster;
		}
		
		unless(defined($globalQueue)){
		    $UseResource = $globalCluster if(defined($globalCluster));
		    $Tip = "!\n! You can specify a queue name to run $ARG for resource $UseResource\n!";
		    my $Qname = get_user_prompt("Do you want to execute $ARG on a spcecific queue ?", 'n', '(y|n)', $Tip);
		    
		    #Use specific queue name?
		    if($Qname eq 'y'){
			$Qname = get_user_prompt("Enter the queue name to run $ARG using resource $UseResource", $queuename, '\S+');
			chomp($Qname);
			$applConf->{$ARG}->{'queue.name'} = $Qname;
			$queuename = $Qname;
		    }
		}else{
		    $applConf->{$ARG}->{'queue.name'} = $globalQueue;
		}
		
	    } #end $UseQ?
	    else{
		
		unless($seen){
		    $Tip = "!\n! PLEASE NOTE:\n"
		 	."! InterProScan can be launched through a cluster machine using RSH. To be able to utilise this\n"
		    	."! feature, all your machines must be visible to each other (see the .rhost file) and be mounted on\n"
			."! a shared file system, such as NFS\n"
			."! (If you are installing InterProScan for use on only one machine please ignore the above message)\n!\n";

		    print STDOUT $Tip;
		    $seen = 1;
		}
		my $res_server;
		undef($res_server);
		my $res_arch;
		
		SERVER : while(!defined($res_server)){
		    
		    $res_server = get_user_prompt("\nPlease enter the execution host name of $ARG", $serv, '\S+');
		    chomp($res_server);
		    if($local_server ne $res_server){
			# rsh command has different behavior depending on the shell type.
			
			chomp(my $shell = $ENV{SHELL});
			my $test;
			my $err = "$HOME/rsh_err";
			
			print STDOUT ">>Testing command : rsh $res_server \"cd $HOME; hostname\" 2> $err\"\n";
			if($shell =~ /csh/){
			    $test = `rsh $res_server \"cd $HOME\;hostname\" 2> $err`;
			}else{
			    $test = `rsh $res_server \"cd $HOME\; hostname\" 2>$err`;
			}

			if(-s $err > 0){
			    $test = `cat $err`;
			    $test =~ s/\n//g;
			}
			unlink($err);
			
			if ($test =~ /$HOME\s*:\s*No such file/i)
			  {
			      print STDOUT "WARNING: \"$HOME\" is not visible from \"$res_server\"?\n";
			      undef($res_server);
			      next SERVER;
			  }
			elsif ($test =~ /Login incorrect/i)
			  {
			      print STDOUT "WARNING: You are  not allowed to login on $res_server\n";
			      undef($res_server);
			      next SERVER;
			  }
			elsif ($test =~ /Unknown host/i)
			  {
			      print STDOUT "WARNING: Unknown host - $res_server\n";
			      undef($res_server);
			      next SERVER;
			  }
			elsif ($test =~ /Permission denied/i)
			  {
			      print STDOUT "WARNING: Permission denied: it's possible the host is not included in your .rhosts file, to fix it type:\n";
			      print STDOUT "echo \"$res_server `whoami`\" >> ~/.rhosts; chmod go-rwx ~/.rhosts <enter>\n";
			      print STDOUT "If that didn't help, consult your system administrator\n";
			      undef($res_server);
			      next SERVER;
			  }
			elsif ($test =~ /Host is unreachable/i){
			      print STDOUT "WARNING: The server $res_server does not respond. Maybe it is down or simply unreacheable\n";
			      print STDOUT "Contact your system administrator or try another host\n";
			      undef($res_server);
			      next SERVER;
			}
			elsif ($test =~ /^\s*$/){
			      print STDOUT "WARNING: No response has been returned by $res_server. Maybe the server is down or simply unreachable\n";
			      print STDOUT "Contact your system administrator or try another host\n";
			      undef($res_server);
			      next SERVER;
			}
			
			$res_arch = `rsh $res_server uname`; #chomp $arch;
			if ($res_arch =~ /(\S+)\s*$/s){	#in case rsh returns a mess
			    $res_arch = $1;
			}
		    }  #end if
		    else{
			$res_arch = $local_arch;
		    }
		    
		    if($res_arch !~ /$supported_arch/){
			print STDOUT "WARNING: The \"$res_arch\" UNIX OS ($res_server) is not supported in this distribution, try another host.\n";
			undef($res_server);
			next SERVER;
		    }
		    
		    #Binaries is symlink to the arch on the execution server.
		    my $lsbin = "$HOME/bin/binaries";
		    
		    if($res_server eq $serv){
			unless(-l $lsbin){
			    unless(-d "$HOME/bin/$res_arch"){
				print STDOUT "WARNING: The \"$res_arch\" UNIX OS ($res_server) is supported but the binaries directory $res_arch\n"; 
				print STDOUT "is NOT installed. Please install it under the same path than your InterProScan installation:\n";
				print STDOUT "$HOME/bin/$res_arch\n";
				print STDOUT "Then create a symbolic link called binaries to this $res_arch directory\n";
				print STDOUT "e.g.: cd $HOME/bin; ln -s $res_arch binaries\n";
				print STDOUT "(check ftp.ebi.ac.uk/pub/databases/interpro/iprscan/BIN for the binaries) or try another host\n";
				print STDOUT "You will have to do it for each server having different architecture than the install host ($local_server).\n";
				undef($res_server);
				next SERVER;
			    }else{
				my $res = `cd $HOME/bin; ln -s $res_arch binaries`;
				if($CHILD_ERROR){
				    print STDOUT "WARNING: Unable to create symbolic link 'ln -s $res_arch binaries': $!\n";
				    print STDOUT "Please correct the error or create the symbolic link manually if necessary\n\n";
				    undef($res_server);
				    next SERVER;
				}
			    }
			} #end else 1
		    } #end if res_server eq serv
		    else{
			my $err = "$HOME/rsh_err";
			my $res = `rsh $res_server \"ls -l $HOME/bin/$res_arch\" 2>$err`;
			
			if(-s $err > 0){
			    $res = `cat $err`;
			    $res =~ s/\n//g;
			}
			unlink($err);
			
			if ($res =~ /$HOME\/bin\/$res_arch\s*:\s*No such file/i)
			  {
			      print STDOUT "WARNING: \"$HOME\/bin\/$res_arch\" does not exist on \"$res_server\"?\n";
			      print STDOUT "Did you install the binaries directory ($res_arch) on your server [$res_server]?\n";
			      print STDOUT "If not, please download the binaries tarball from ftp.ebi.ac.uk/pub/databases/interpro/iprscan/BIN\n";
			      print STDOUT "and install it. Otherwise, check if the path \"$HOME/bin/$res_arch\" is visible from your $local_server\n";
			      undef($res_server);
			      next SERVER;
			  }
			$res = `rsh $res_server \"ls -l $HOME/bin/binaries\" 2> $err`;
			
			if(-s $err > 0){
			    $res = `cat $err`;
			    $res =~ s/\n//g;
			}
			unlink($err);
			
			if($res =~ /$HOME\/bin\/binaries\s*:\s*No such file/i){
			    print STDOUT "\n>>Creating symbolic link \"$res_arch -> binaries\" on server $res_server ... \n";
			    $res = `rsh $res_server \"cd $HOME/bin; ln -s $res_arch binaries\" 2>$err`;
			    if(-s $err > 0){
				$res = `cat $err`;
				$res =~ s/\n//g;
				print STDOUT "WARNING: creating symlink produced error \"$res\"\n";
			    }
			    print STDOUT "\n>>DONE\n";
			}
			unlink $err;
		    } #end else
		    
		}#end while
		$applConf->{$ARG}->{exechost} = $res_server;
		$applConf->{$ARG}->{queue} = 'local';
		$serv = $res_server;
	    } #end else $UseQ eq y
	} #end use appl eq y
	else{
	    $applConf->{$ARG}->{run} = 0;
	}
    } #end for @applications

    $Tip = "!\n! InterProScan can use taxonomy-based filtering of results which uses the InterPro taxonomy listed for each InterPro\n"
	."! entry to hide hits not relevant to that taxon.\n!";

    $usetaxo = get_user_prompt("Do you want to make it available for users?", 'y', 'y|n', $Tip);
    $usetaxo = ($usetaxo eq 'y' ? '1' : '0');

    $Tip = "!\n! You can specify an email address for the administrator. By default it will be you ($ENV{USER})\n"
	."! Administrators receive an email when a problem occurs in InterProScan.\n!";

    $user = get_user_prompt("Do you want to set an administrator email address?", 'y', 'y|n', $Tip);
    if($user eq 'y'){
	$user = get_user_prompt("Please enter the email address of the administrator:", '', '[\w\.\-]+\@[\w\.\-]+');
    }else{
	$user = '';
    }
    
    print STDOUT ">>Writing to the configuration files ... \n";
    
    foreach my $key (sort keys %{$applConf}){
	
	unless($applConf->{$key}->{run}){
	    delete($applConf->{$key});
	    next;
	}
	my $file = $key;
	$file .= ".conf";
	$file = "$HOME/conf/" . $file;
	die "FATAL: File $file does not exist: $!" unless (-f $file);
	
	patch_file($file, 'adminaddr', $user, 1);
	patch_file($file, 'mailfromaddr', $user, 1);
	
	print STDOUT ">>Processing file : $file ... ";
	if($applConf->{$key}->{usequeue}){
	    patch_file($file, 'queue', $applConf->{$key}->{queue},1);
	    patch_file($file, 'resource', $applConf->{$key}->{resource}, 1);
	    patch_file($file, 'queue.name', $applConf->{$key}->{'queue.name'}, 1);
	}else{
	    patch_file($file, 'queue', $applConf->{$key}->{queue}, 1);
	    patch_file($file, 'host.exec', $applConf->{$key}->{exechost}, 1);
	}
	print STDOUT ">>DONE\n";
    }
    
} #end if($applset eq 'y')

unless(-f "$HOME/conf/iprscan.conf"){
    
    print STDOUT "WARNING: You don't seem to have a configuration file: $HOME/conf/iprscan.conf\n";
    print STDOUT "If you wish, you can manually create and edit this file (it is critical it exists for InterProScan to function)\n";
    print STDOUT "Ensure you have the following set in the file:\n";
    print STDOUT "\tqueue=", $UseQ eq 'y' ? 'your queueing system' : 'local',"\n";
    print STDOUT "\tresource=The cluster name where to run iprscan if needed\n" if $UseQ eq 'y';
    print STDOUT "\tqueue.name=The queue name you want iprscan to use\n" if $UseQ eq 'y';
    print STDOUT "\thost.exec=hostname you want to run iprscan (~master to launch applications)\n" if $UseQ eq 'n';
    print STDOUT "\tapplications=",join(',', @setAppl),"\n";
    print STDOUT "\tchunk=$chunkz\n";
    print STDOUT "\tmaxinputseqs.aa=$maxinaa\n" if(defined($maxinaa));
    print STDOUT "\tmaxinputseqs.nt=$maxinnt\n" if(defined($maxinnt));
    print STDOUT "\tmaxseqlen.nt=$maxlennt\n" if(defined($maxlennt));
    print STDOUT "\tminseqlen.aa=$minlenaa\n" if(defined($minlenaa));
    print STDOUT "\tminorfsize=$minorflen\n" if(defined($minorflen));
    print STODUT "\tcodon.table=$cdntable\n" if(defined($cdntable));
    print STODUT "\ttaxonomy.use=$usetaxo\n" if($usetaxo);
    print STDOUT "\n";
}else{
    print STDOUT ">>Setting File $HOME/conf/iprscan.conf ... ";
    patch_file("$HOME/conf/iprscan.conf", 'queue', $Qsys ? $Qsys : 'local' , 1);
    patch_file("$HOME/conf/iprscan.conf", 'applications', join(',', @setAppl), 1);
    patch_file("$HOME/conf/iprscan.conf", 'resource', $globalCluster ? $globalCluster : '', 1);
    patch_file("$HOME/conf/iprscan.conf", 'queue.name', $globalQueue ? $globalQueue : '', 1);
    patch_file("$HOME/conf/iprscan.conf", 'host.exec', $Qsys ? '' : $local_server, 1);
    patch_file("$HOME/conf/iprscan.conf", 'adminaddr', $user ? $user : '', 1);
    patch_file("$HOME/conf/iprscan.conf", 'mailfromaddr', $user ? $user : '', 1);
    patch_file("$HOME/conf/iprscan.conf", 'chunk', $chunkz ? $chunkz : '10', 1);
    patch_file("$HOME/conf/iprscan.conf", 'maxinputseqs.aa', $maxinaa, 1) if(defined($maxinaa));
    patch_file("$HOME/conf/iprscan.conf", 'maxinputseqs.nt', $maxinnt, 1) if(defined($maxinnt));
    patch_file("$HOME/conf/iprscan.conf", 'maxseqlen.nt', $maxlennt, 1) if(defined($maxlennt));
    patch_file("$HOME/conf/iprscan.conf", 'minseqlen.aa', $minlenaa, 1) if(defined($minlenaa));
    patch_file("$HOME/conf/iprscan.conf", 'minorfsize', $minorflen, 1) if(defined($minorflen));
    patch_file("$HOME/conf/iprscan.conf", 'codon.table', $cdntable, 1) if(defined($cdntable));
    patch_file("$HOME/conf/iprscan.conf", 'taxonomy.use', $usetaxo, 1) if($usetaxo);
    print STDOUT ">>DONE\n";
}
 

$Tip = "!\n! InterProScan can be launched through a web interface. It can also be launched through secure HTTP (https)\n!";

my $useWeb = get_user_prompt("Do you want to run InterProScan using a web interface?", 'n', 'y|n', $Tip);
my $HTTPProto;
if($useWeb eq 'y'){
    
    $HTTPProto = get_user_prompt("Do you want to run InterProScan through a secure HTTP server?", 'n', 'y|n');
    
    $HTTPProto = ($HTTPProto eq 'y' ? 'https://' : 'http://');
    
    my $workserver = get_user_prompt("Enter the name of your server. You can specify a specific port to listen to. (e.g. \"foo.bar.com:4000\")", 'foobar.com', '[\w\.\_\-\:\d]+');
    if($workserver =~ /^\s*(https?:\/\/)/){
	$workserver =~ s/$1//;
    }
    $HTTPProto .= $workserver;
    patch_file("$HOME/conf/iprscan.conf", 'workserver', $HTTPProto, 1);
    my ($restdom) = ($HTTPProto =~ /[^.]+(\.[^:\/\n\s]+)/);
    print STDOUT "!\n! Please add the following lines to your web server configuration (httpd.conf):\n"
               . "#--------------------------------------\n"
	       . " Alias /doc/ \"$PWD/doc/html/\"\n"
	       . " Alias /images/ \"$PWD/images/\"\n"
	       . " Alias /tmp/ \"$PWD/tmp/\"\n\n"
	       . " ScriptAlias /iprscan/ \"$PWD/bin/\"\n\n"
	       . " <Directory \"$PWD/images\">\n"
               . "   Options Indexes MultiViews\n"
               . "   AllowOverride None\n"
               . "   Order allow,deny\n"
               . "   Allow from all\n"
               . " </Directory>\n\n"
	       . " <Directory \"$PWD/tmp/\">\n"
	       . "   Options FollowSymLinks Includes SymLinksIfOwnerMatch\n"
	       . "    IndexIgnore */.??* *~ *# */HEADER* */README* */RCS\n"
	       . "    AllowOverride AuthConfig Limit FileInfo\n\n"
	       . "    Order deny,allow\n"
	       . "    Deny from all\n"
	       . "    Allow from $restdom\n"
	       . " </Directory>\n"
	       . "#--------------------------------------\n";
    print STDOUT "! Be sure that 'apache' or 'httpd' virtual user (uid/gid) have right to read/write into your $PWD iprscan directory.\n";
    print STDOUT "! Otherwise, if you are running a standalone http(s) web server, be sure also the user running it have to enough\n";
    print STDOUT "! rights to read/wrtie $PWD iprscan directory\n";
    print STDOUT "! If you encoutered problem with your web server running iprscan, read the FAQ or contact your system administrator.\n\n";
    print STDOUT "! After restarting your web server try $HTTPProto/iprscan/iprscan\n!\n";
    
    sleep(10);
    
}


#Indexation of datafiles
print STDOUT ">>Indexing datafile ... ";
unless(-f "$HOME/bin/index_data.pl"){
    print STDOUT "WARNING: Config.pl is unable to index your datafiles as it appears that the index script is not in your installation!\n";
    print STDOUT "Go to your iprscan/bin directory and check if the script index_data.pl is there and executable.\n";
    print STDOUT "You can index them separately after finishing your installation.  For help, type './index_data.pl' -h\n";
}else{
    
    if(system("$PERL_CMD $HOME/bin/index_data.pl -inx -bin 2> $HOME/bin/inx.errors")){
	print STDOUT "WARNING: Some problems occured when trying to index the data files.\n";
	if(-s "$HOME/bin/inx.errors" > 0){
	    unless(open(ERR, "<$HOME/bin/inx.errors")){
		print STDOUT "Could not open $HOME/bin/inx.errors : $!\n";
	    }else{	
		print STDOUT " Please fix the error below and index your datafiles using the $HOME/bin/index_data.pl script.\n";
		print STDOUT " For help, type './index_data.pl -h' \n";
		while(<ERR>){
		    print STDOUT "errormsg: $_";
		}
		close ERR;
	    }
	}
	
    }
    unlink("$HOME/bin/inx.errors");
    print STDOUT ">>DONE\n";
}


####### Fill in registration
$Tip = "!\n!  We would appreciate it if you would register your installation of InterProScan.  This will allow us to notify you\n"
	."! of bug fixes and new releases of the software. We will not use your email address for any other purpose.\n!";

my $register = get_user_prompt("Would you like to register InterProScan?", $insDone, '(y|n)', $Tip); 
if ($register eq "y")
  {
      my $org_name = get_user_prompt("What is the name of your organization?", '', '\S+'); 
      my $person = get_user_prompt("Who is the contact person?", '', '\S+'); 
      my $email = `whoami`;
      chomp $email;
      $email .= '@'.`hostname`;
      chomp $email;
      $email = get_user_prompt("What is the e-mail address?", $email, '\S+'); 
      $HTTPProto = get_user_prompt("If you installed the web interface, what is the server name?", $HTTPProto, '(\S+|n)'); 
      
      my $mbody = "$org_name\\|$person\\|$email\\|$HTTPProto";
      $mbody =~ s/\n/ /;
      if ( -x "/usr/bin/Mail" )
	{
	    $mbody =~ s/\@/\\\@/;
	    `echo $mbody | /usr/bin/Mail -s 'InterProScan registration.' interhelp\@ebi.ac.uk`; 
	    `echo "subscribe interproscan-announce $email" | /usr/bin/Mail majordomo\@ebi.ac.uk`; 
	}
      else
	{
	    $mbody =~ s/\\//g; 
	    print STDOUT "\n    Please, email the following message\n"
	               . "    to:interhelp\@ebi.ac.uk\n"
		       . "    sbj:InterProScan registration.\n"
		       . "    msg:$mbody\n\n"
		       . "    and, email the following message to get the update announces\n"
		       . "    to:majordomo\@ebi.ac.uk:\n"
		       . "    msg:subscribe interproscan-announce $email\n\n";
	}
      print STDOUT "Thank you.\n";
  }

print STDOUT "\n>>Installation is completed.  Please test your installation by running on the command line:\n"
	."\n\t$HOME/bin/iprscan -cli -i $HOME/test.seq -o $HOME/test.out -format raw -goterms -ipr\n\n"
	."and comparing it with the merged.raw file that is supplied."
	."\nFor help and options on the commandline, run $HOME/bin/iprscan -cli -h\n\n";



###############
## Subroutines#
###############

####################################################################################################
# Get and return the user answer $input to the question $prompt with suggestion $suggest, 
# valid pattern $valid and tip $Tip
####################################################################################################
sub get_user_prompt
  {
      my ($prompt, $suggest, $valid, $Tip) = @_;
      my ($input) = (undef);
      
      print STDOUT "\n! Tip:\n$Tip\n" if ( defined($Tip) && ($Tip !~ /^\s*$/) );
      while ( !defined($input) )
	{
	    print STDOUT "    $prompt [$suggest] /$valid/? : ";
	    $input = <>;
	    chomp($input);
	    $input = $suggest if ( $input =~ /^\s*$/ );	# empty input => user was happy with the suggestion
	    if ( $input !~ /^$valid$/ )
	      {
		  print STDOUT "WARNING: The information input is not valid (doesn't match /$valid/), please try again.\n";
		  undef($input);
	      }
	}
      return($input);
  }  # end get_user_prompt


####################################################################################################
# Replace the value of  the first occurence of variable called $var_name by $value in file $file
# WARNING: $var_name must be at the beginning of the line (^\s*$var_name\s*=\s*)
####################################################################################################
sub which_binaries
  {
      my ($print_name, $file_name, $version_opt, $required_version, $Tip) = @_;
      my ($prompt, $input) = ("", "", "");
      
      print STDOUT "\n! Tip:\n$Tip\n" if ($Tip);
      if ( ! -x $file_name )
	{
	    $prompt = "Please enter the full path of your $print_name installation";
	}
      else
	{
	    $prompt = "Should I use $print_name installation [$file_name]? ";
	}
      
      undef($input);
      while ( !defined($input) )
	{
	    print STDOUT "    $prompt: ";
	    $input = <>;
	    chomp($input);
	    $input = $file_name if ( $input =~ /^\s*$/ );	# empty input => user was happy with the suggestion
	    if  ( ! -x $input )
	      {
		  print STDOUT "WARNING: What you have input is not an executable, try again.\n";
		  undef($input);
		  next();
	      }
	    my $grep_cmd = '/bin/grep';
	    $grep_cmd = '/usr/bin/grep' if ($^O =~ /darwin/i);
	    $grep_cmd = '/usr/xpg4/bin/grep' if ($^O =~ /solaris/);
	    
	    #$grep_cmd = "grep" if ( !defined($grep_cmd) || ($grep_cmd !~ /\S/) );
	    unless ( `$input $version_opt 2>&1 1| $grep_cmd -cE "$required_version"` > 0 )
	      {
		  print STDOUT "WARNING: The version is incorrect (doesn't match /$required_version/), try again.\n";
		  undef($input);
		  next();
	      }
	}
      return($input);
  }  # end which_binaries

####################################################################################################
# Replace the value of  the first occurence of variable called $var_name by $value in file $file
# WARNING: $var_name must be at the beginning of the line (^\s*$var_name\s*=\s*)
####################################################################################################
sub patch_file
  {
      my ($file, $var_name, $value, $quote) = @_;
      my ($items) = ("");
      my (@Tab_file) = ();
      
      if ( ! open(ORI_FILE, "$file") )
	{
	    print STDOUT "WARNING: Can't open file '$file' : $!\n";
	    exit(1);
	}
      @Tab_file = <ORI_FILE>;
      close(ORI_FILE);
      
      foreach my $i (0 .. $#Tab_file)
	{
	    my $line = $Tab_file[$i];
	    
	    if ( $line =~ /^\s*($var_name\s*=)/ )
	      {
		  chomp($line);
		  $line = "${1}\'$value\';\n";
		  $line =~ s/[\'\;]//g if($quote);
		  $Tab_file[$i] = $line;
		  last();
		}
	}
      if ( ! open(RES_FILE, ">$file") )
	{
	    print STDOUT "WARNING: Can't open file '$file' in write mode : $!\n";
	    exit(1);
	}
      print RES_FILE @Tab_file;
      close(RES_FILE);
  }  # end patch_file
