#!/ebi/extserv/bin/perl/bin/perl

# This program parses raw fingerprintscan output into the format required by Alex K 
# i.e. seq_id|prints_ac|prints_id|start|end|evalue

# For Paul : Florence's modifcations (Fri Jan 24 10:09:39 CET 2003)
#   - option -f can be replaced by giving the fps file on STDIN to this program
#       (usage of the program has been modified accordingly)
#   - a record separator is printed in the output file (see $RECORD_SEP)


use Getopt::Std;

($opt_d, $opt_f) = (undef, undef);
$RECORD_SEP = "\/\/\n";


getopts('d:f:');

#check command line for correct number of arguments
die print "you must enter a database!

  USAGE:
	'$0 -d <database_file> -f <fps.out>  > fps.out.final'
    OR
	'cat <fps.out> | $0 -d <database_file>  > fps.out.final'\n\n" if ( !defined($opt_d) );



#########read in multi-dimensional database######

$database_name = $opt_d;

unless(open (DB, "$database_name")){
  die "Could not open $database_name : $!";
}

%label = (

       0 => "accession",
       1 => "Evalue",
       2 => "motifs",
       3 => "siblings",

);

while (<DB>) {

    chomp;
  @array = split /\|/;

  foreach (1..4) {

    $HOH {$array[0]} { $label{$_-1} } = $array[$_];

  }

}
##################################################

close DB;

$file_name = $opt_f;
@file = ();

if ( defined($file_name) )
{
	# Redefine our own stdin if the user used the -f option:
	close(STDIN);
	unless(open (STDIN, "$file_name")){ die "Could not open $file_name : $!";}
}

while (<STDIN>) {

  # vsi 2003-12-16: this causes problems when the input sequence
  # header contains characters ';' or ',', so read until space
  #if (/Sn; ([^\s\;,]+)/){ #beginning of a new entry
  if (/Sn; ([^\s]+)/){ #beginning of a new entry

    #re-initialise variables
    %print = ();
    @temp_array = ();
    $sptr = $1;
    @siblings = ();
    %siblings = ();
    %keep = ();
  }

#make allowances for some 2-element fingerprints

  if (/2TB(N|H)/) { #read all entries from table 2

    @temp_array = split /\s+/;
    #assign PRINTS ID and E-value to a hash if the E-value and number of motifs matched pass the 
    #specifications in the db file
    $print{$temp_array[1]} = "$temp_array[9]" if $temp_array[9] < $HOH {$temp_array[1]} {Evalue} 
      and $temp_array[2] >= $HOH {$temp_array[1]} {motifs};

  }

  @temp_array = ();

  if (/3TB(N|H)/) { #read entries from table 3

    @temp_array = split /\s+/;
    #calculate match start and end values
    $start = $temp_array[11];
    $end = ($start + $temp_array[9]) - 1;
    #create an array of id&%&PR0****&%&id&%&start&%&end&%&evalue for each motif matched if the id exists 
    #in the print hash
    push (@file, "$sptr&%&$HOH{$temp_array[1]}{accession}&%&$temp_array[1]&%&$start&%&$end&%&$print{$temp_array[1]}\n") 
      if exists $print{$temp_array[1]};

  }

  if (/3TBF/) { #parse file array for siblings at the end of table 3

     foreach (@file) {

	chomp;
	@columns = split /&%&/;
	$current_value = $columns[0];
	$prints_code = $columns[2];

	%siblings = retrieve_siblings($prints_code);

	#go to next entry if current code exists in the siblings table
	next if exists $siblings{$prints_code};

	#otherwise print entry and push it into "keep" hash
	print "$current_value&%&$columns[1]&%&$prints_code&%&$columns[3]&%&$columns[4]&%&$columns[5]\n";

	$keep{$prints_code} = "keep_this"; 

     }
     print "$RECORD_SEP" if ( $#file >= 0 );
     @file = ();

   }

}

close(STDIN);

##################subroutines####################

sub retrieve_siblings {

  return %siblings if exists $keep{$prints_code};
  return %siblings if ! ($HOH{$prints_code}{siblings});
  $scalar_siblings = $HOH{$prints_code}{siblings};
  @array_siblings = split ( /\,/, $scalar_siblings);

  foreach (@array_siblings) {

    $siblings{$_} = "sib";

  }

  return %siblings;

}

#################################################
