#!/ebi/extserv/bin/perl/bin/perl


=head1 NAME

ResubmitJobs.pl - ReSubmission script if failed jobs for CGI and CLI.

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 VERSIONS

$Id: ResubmitJobs.pl,v 1.1.1.1 2005/08/18 13:18:25 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut



BEGIN {
$ENV{IPRSCAN_HOME}='/scratch/tmp/iprscan';
}

use CGI qw(:standard :shotcuts);
use CGI::Carp qw(fatalsToBrowser);
use Data::Dumper;
use English;
use Getopt::Long;
use File::Basename;
use strict;

use lib "$ENV{IPRSCAN_HOME}/lib";
use Dispatcher::Tool;
use Dispatcher::Tool::InterProScan;
use Dispatcher::Queue;
use Index;

$| = 1;

my($res, $msg);
my $params = { };
my $html = 0;
my $verbose = 0;
my $h_chunk = { };
my $file;
my $out;
my $query = new CGI();

my $iprscan = Dispatcher::Tool::InterProScan->new();
$iprscan->setToolName('iprscan');

if($ARGV[0] || !$query->param()){
  my ($help);

  unless(&GetOptions('r:s' => \$file,
		     'h'   => \$help,
		     'o:s' => \$out,
		     'v:i' => \$verbose)){
    usage();
    exit;
  }

  if($help || !$file){
    usage();
    exit;
  }

  unless(-f $file){
    handleError("$file does not exist : $!", undef, 1);
  }

  my $pfile = $file;
  $pfile =~ s/report$/params/;

  #Reading param file we can retrieve the workdir from DDMMYYY etc..
  ($res, $msg) = $iprscan->readParamFile($pfile, $params);
  handleError($msg, undef, 1) unless($res);

  $iprscan->setParams($params);
  $iprscan->setParam('cmd', 'submit');
  $iprscan->setParam('srchtype', 'interactive');
  $iprscan->setParam('submission', 'cli');


}else{
  $html = 1;
  
  if($query->param()){

    ($res, $msg) = $iprscan->parseRequest($query);
    $iprscan->errorExit($msg, undef, 1) unless($res);

    $iprscan->setToolName($iprscan->getParam('tool'));
    $iprscan->setParam('submission', 'cgi');

    #This parameter is set when user press the 'Resubmit' button.
    if($iprscan->getParam('toolparams')){
      ($res, $msg) = $iprscan->readParamFile($iprscan->getParam('toolparams'), $iprscan->getParams());
      $iprscan->errorExit($msg, undef, 1) unless($res);
      $iprscan->removeParam('toolparams');
      $iprscan->setParams($iprscan->getParams());
      $iprscan->setParam('cmd', 'submit');
    }else{
	$iprscan->setParam('cmd', 'check');
    }

  }else{
    $iprscan->errorExit("No parameters found in the CGI query!", undef, 1);
    exit;
  }
}

($res, $msg) = $iprscan->parseConfig(1, $iprscan->getParams(), 1);
$iprscan->errorExit($msg, undef, 1) if (!$res && $html);

#CGI need to read report file each time the page is refreshed to keep in memory values of h_chunk table
if($html){

  $verbose = 2;

  $file = $iprscan->getConfigValue('toolreport');
  unless(-f $file){
    #First time we submit. We create the root report file concatenating all the report from each chunk.
    my $wdir = $iprscan->getConfigValue('workdir');

    if(system("find $wdir -name \\*.report -exec cat {} > $file.tmp \\;")){
      $iprscan->errorExit("[find] command could not be launch or failed : $!", undef, 1);
    }
    if(system("find $wdir -name \\*.report -exec rm {} \\;")){
      $iprscan->errorExit("[find] command could not be launch or failed : $!", undef, 1);
    }

    unless(rename("$file.tmp", $file)){
      $iprscan->errorExit("Could not rename [$file.tmp] to [$file] : $!", undef, 1);
    }
  }
}


#Maybe better to do it here
#Read the concatenated report file.
print STDERR "Reading report file ... " if($verbose);
($res, $h_chunk) = readReportFile($iprscan, $h_chunk, $file);
unless($html){
  handleError($h_chunk, undef, 1) unless($res);
}else{
  $iprscan->errorExit($h_chunk, undef, 1) unless($res);
}
print STDERR "DONE\n" if($verbose);


my $cmd = $iprscan->getParam('cmd');
my $tname = $iprscan->getToolName();
#To tell iprscan_wrapper do not create chunk when you read sequence file. It has been already done previously.
$iprscan->setParam('resubmit', '1');

if($cmd eq 'submit'){

  my $pfile = $iprscan->getConfigValue('toolparams');
  my $toolnocrc = $iprscan->getConfigValue('toolnocrc');
  my $rfile = $iprscan->getConfigValue('toolraw');
  my $efile = $iprscan->getConfigValue('toolerrors');
  my $workdir = $iprscan->getConfigValue('workdir');

###############  Here starts the critical loop !!! ##########################

  foreach my $chunk (sort {$a <=> $b} keys %$h_chunk){

    #We will resubmit job for failed chunk(s)
    ($res, $msg) = $iprscan->addChunk($chunk, \$toolnocrc, \$rfile, \$efile, \$workdir);
    if($html){
      $iprscan->errorExit($msg, undef, 1) unless($res);
    }else{
      handleError($msg, undef, 1) unless($res);
    }

    #We reload the sequence file for this specific chunk
    print STDERR "Uploading $toolnocrc ... " if($verbose);
    $iprscan->setConfigValue('uploadfile.1', "$toolnocrc");
    print STDERR "DONE\n" if($verbose);

    #We need to do it otherwise the command stay 'submit'. Occurs some problems later
    $iprscan->setParam('cmd', 'check');

    #We set the failed applications to run.
    $iprscan->setParam('appl', join(',', map { $_->[0] } @{$h_chunk->{$chunk}->{appl}}));

    #We don't need above cause we are launching all the jobs 

    #We need to rename the previous results to avoid to overwrite them
    if(-f $rfile){
      print STDERR "Renaming raw file $rfile ... " if($verbose);
      unless(rename("$rfile", "$rfile.resub")){
	if($html){
	  $iprscan->errorExit("Unable to rename $rfile to $rfile.resub: $!", undef, 1);
	}else{
	  handleError("Unable to rename $rfile to $rfile.resub: $!", undef, 1);
	}
      }
      print STDERR "DONE\n" if($verbose);
    }else{
      if($html){
	$iprscan->errorExit("$rfile does not exist. Run does finished?", undef, 1);
      }else{
	handleError("$rfile does not exist. Run does finished?", undef, 1);
      }
    }

    #We remove all the indexed output and error to avoid problem when checking jobs
    foreach my $appl (@{$h_chunk->{$chunk}->{appl}}){

      print STDERR "Loop over chunk $chunk:\n" if($verbose > 1);
      my $jobid = $appl->[1];
      #We set the job status as RUNNING to be sure.
      $h_chunk->{$chunk}->{status}->{$jobid} = Dispatcher::Queue::JOB_RUNNING;

      my $inx = $iprscan->getConfigValue('workdir');
      my $erfile = $inx;
      my $exfile = $inx;

      $inx .= "/$jobid.output.inx";
      $erfile .= "/$jobid.errors";
      $exfile .= "/$jobid.exitcode";

      ($res, $msg) = $iprscan->addChunk($chunk, \$inx, \$erfile, \$exfile);
      if($html){
	$iprscan->errorExit($msg, undef, 1) unless $res;
      }else{
	handleError($msg, undef, 1) unless $res;
      }

      print STDERR "Removing files :\n\t$erfile\n\t$exfile\n ... " if($verbose > 1);
      unless(unlink($erfile, $exfile)){
#	if($html){
#	  $iprscan->errorExit("Unable to remove output files : $!", undef, 1);
#	}else{
#	  handleError("Unable to remove output files : $!", undef, 1);
#	}
      }
      print STDERR "DONE\n" if($verbose > 1);
      unlink("$inx") unless($iprscan->getParam('submission' eq 'cli'));
    } #end for loop


    if ($iprscan->isInteractive()) {
      my $jobs = { };
      print STDERR "[$chunk] Submiting Jobs ... " if($verbose > 1);
      ($res, $jobs, $msg) = $iprscan->submitJobs($toolnocrc);
      if($html){
	$iprscan->errorExit($msg, undef, 0) unless $res;
      }else{
	handleError($msg) unless $res;
      }
      print STDERR "DONE\n" if($verbose > 1);
      $h_chunk->{$chunk}->{jobs} = $jobs;
    }else{
      #FIXME unlink the list of files recreated or changed to allow user to launch again resubmit
      if($html){
	$iprscan->errorExit("InterProScan run only in interactvie mode using ResubmitJobs.pl", undef, 1);
      }else{
	handleError("InterProScan run only in interactvie mode using ResubmitJobs.pl", undef, 1);
      }
    }
  }  #end foreach my $chunk

  if($html){
    #We need to write the param file once again otherwise next the time the script is called with the 'check'
    #command it will loose the data given by the Resubmit page, like view ('picture' or 'table').
    my $paramfile = $iprscan->getConfigValue('toolparams');
    print STDERR "\nWriting Params file [$paramfile] ... " if($verbose > 1);
    ($res, $msg) = $iprscan->writeParamFile($paramfile, $iprscan->getParams());
    $iprscan->errorExit($msg, undef, 1) unless($res);
    print STDERR "DONE\n" if($verbose > 1);
    my $url = $iprscan->getConfigValue('resuburl');
    $url .= '1';
    print redirect(-location => $url, -method => "PUT");
    exit;
  }

  my $pstart = 0;
  my $rstart = 0;

  #We can now remove the main report file.
  unlink($file);

  #To store reports job if jobs failed again.
  my $reporton = 0;

  #We check jobs until they are all done.
  while (1) {

    #We must loop over each keys (chunk) otherwise we will have problem
    #if the failed jobs don't follow each other (e.g. : 1, 3,4,14,18 ..)
    #See Dispatcher::Tool::InterProScan::checkJobs sub to understand better how it works.

    my $alldone = 1;
    foreach my $chunk (sort {$a <=> $b} keys %$h_chunk){
      print STDERR "[$chunk] Chekcing job ... " if($verbose > 1);
      ($res, $h_chunk) = $iprscan->checkJobs($h_chunk, $chunk);
      handleError("failed to check jobs : $h_chunk") unless $res;
      print STDERR "DONE\n" if($verbose > 1);

      ($res, $msg) = $iprscan->addChunk($chunk, \$workdir);
      handleError($msg) unless($res);

      my $report = $h_chunk->{$chunk}->{status};
      my $jobs   = $h_chunk->{$chunk}->{jobs};

      foreach my $jobid (keys %$report){

	my $repfile = "$workdir/$jobid.report";
	print STDERR "[$chunk] checking status $jobid ... " if($verbose > 1);
	my $status = $report->{$jobid};
	print STDERR " => $status\n" if($verbose > 1);

	if ($status == Dispatcher::Queue::JOB_PENDING) {
	  unless ($pstart) {
	    print "\nPENDING   " if($verbose);
	    $pstart = 1;
	  }
	  $alldone = 0;
	  print "." if($verbose);
	}
	elsif ($status == Dispatcher::Queue::JOB_RUNNING) {
	  unless ($rstart) {
	    print "\nRUNNING   " if($verbose);
	    $rstart = 1;
	  }
	  $alldone = 0;
	      print "." if($verbose);
	}
	elsif ($status == Dispatcher::Queue::JOB_DONE) {
	  print STDERR "\t[$chunk] $jobid DONE [s:$status]\n" if($verbose > 1);
	  #We need to check if there is no error. Sometime the main program exits
	  #reporting some errors and the exitcode in the .exitcode file is 0.
	  
	  my $exfile = $jobs->{$jobid}->{exitcode};
	  my $erfile = $jobs->{$jobid}->{errors};
	  my $oufile = $jobs->{$jobid}->{output};
	  if(-f $exfile && -s $exfile > 0 ){
	    print STDERR "\t[$chunk] exfile here [$exfile]\n" if($verbose > 1);
	    unless(open(EX, "<$exfile")){
	      return(0, "could not open $exfile : $!");
	      #handleError($tool, "could not open $exfile : $!", $efile, undef, 1);
	    }
	    local $INPUT_RECORD_SEPARATOR;
	    my $errcode = <EX>;
	    close EX;
	    $INPUT_RECORD_SEPARATOR = "\n";
	    print STDERR "\t[$chunk] Excode = $errcode\n" if($verbose > 1);
	    unless(defined($errcode) && $errcode eq "0"){
	      $h_chunk->{$chunk}->{status}->{$jobid} = Dispatcher::Queue::JOB_FAILED;
	      if(-f $erfile && -s $erfile > 0){
		print STDERR  "\t[$chunk] $jobid FAILED\n" if($verbose > 1);
	      }
	    }else{
	      if(-f $erfile && -s $erfile > 0 && -s $oufile == 0){
		print STDERR  "\t[$chunk] $jobid FAILED\n" if($verbose > 1);
		$h_chunk->{$chunk}->{status}->{$jobid} = Dispatcher::Queue::JOB_FAILED;
		print STDERR "\t[$chunk] Creating failed report [$repfile] for $jobid ... "if($verbose > 1);
		($res, $msg) = $iprscan->createFailedReport($jobid, $h_chunk, $repfile);
		handleError($msg) unless $res;
		$reporton = 1;
		
	      }else{
		print STDERR  "\t[$chunk] $jobid DONE\n" if($verbose > 1);
	      }
	    }
	  }else{
	    print STDERR "\t[$chunk] WARNING Could not find $exfile !!!!!!\n" if($verbose > 1);
	  }
	}
	elsif ($status == Dispatcher::Queue::JOB_FAILED) {

	  print STDERR "\n\t[$chunk] $jobid FAILED\n" if $verbose;

	  print STDERR "\t[$chunk] Creating failed report [$repfile] for $jobid ... "if($verbose > 1);
	  ($res, $msg) = $iprscan->createFailedReport($jobid, $h_chunk, $repfile);
	  handleError($msg) unless $res;
	  $reporton = 1;
	  print STDERR "DONE\n" if($verbose > 1);
	  unless(open(ERR, ">>$efile")){
	    close ERR;
	    handleError("Could not open $efile : $!");
	  }
	  print ERR "$jobid FAILED\n";
	  close ERR;
	}
	elsif ($status == Dispatcher::Queue::JOB_NOT_FOUND) {
	  print STDERR "\n\tNOT FOUND\n" if($verbose > 1);
	}
#	print STDERR "\n" if($verbose);
      } #end foreach muy $jobid
    } #end foreach my $chunk
    sleep(1);
    print STDERR "---------------------\n\n" if($verbose > 1);
    last if($alldone);
  } #end while(1)


  #We check if some jobs failed for this chunk
  if($reporton){
    print STDERR "Some Jobs failed. Concatenate them in the root directory ... " if($verbose);
    my $report = $iprscan->getConfigValue('toolreport');
    my $wdir = $iprscan->getConfigValue('workdir');
    if(system("find $wdir -name \\*.report -exec cat {} > $report.tmp \\; -exec rm {} \\;")){
      handleError("Could not concatenate all report files in the root directory : $!", undef, 1);
    }
    unless(rename("$report.tmp", $report)){
      handleError("Could not rename [$report.tmp] to [$report] : $!");
    }
    print STDERR "DONE\n" if($verbose);

    if(-s $report > 0){
      print STDERR "\n\n=== ********************************************************** ===\n";
      print STDERR "=== Some jobs failed! Please check the report file\n";
      print STDERR "=== $report\n";
      print STDERR "=== You can try to relaunch failed jobs using ResubmitJobs.pl\n";
      print STDERR "=== script located in your iprscan/bin directory as follow :\n";
      print STDERR "=== ./ResubmitJobs.pl -r $report\n";
      print STDERR "=== Then old results will be concatenated with new one.\n";
      print STDERR "=== ********************************************************** ===\n\n\n";
    }
  }


  #we have to put here the way to recreate, and parse the results.
  ($res, $msg) = redoResults($iprscan, $h_chunk, $rfile);
  handleError($msg, $efile) unless $res;

  #Now that every chunk have their own raw file, we need to concatenate all of them in the root directory

  my $wdir = $iprscan->getConfigValue('workdir');
  my $rootraw = $iprscan->getConfigValue('toolraw');

  if(system("find $wdir -name \\*.raw -exec cat {} > $rootraw.tmp \\;")){
    handleError("Could not execute [find] command to merge all .raw file : $!", undef, 1);
  }
  unless(rename("$rootraw.tmp", $rootraw)){
    handleError("Could not rename [$rootraw.tmp] to [$rootraw] : $!");
  }
  
  my $format = $iprscan->getParam('format');
  my $ext = 'tool'.$format;
  $ext = 'toolxml' unless ($format);

  my $xmlcmd = $iprscan->getConfigValue('iprscan.xml.converter');
  handleError("iprscan.xml.converter not configured", $efile) unless $xmlcmd;
  
  handleError("no format!") unless($format);
  $xmlcmd =~ s/\%format/$format/;
  $xmlcmd =~ s/\%toolraw/$rootraw/;

  my $resfile = $iprscan->getConfigValue($ext);
  
  system("$xmlcmd > $resfile.tmp 2> $efile");
  handleError("failed to create $format output", $efile) if(-s $efile > 0);
  
  unless (rename("$resfile.tmp", $resfile)) {
    handleError("failed to rename $resfile : $!", $efile);
  }

  if (-s $resfile == 0 && -s $efile > 0) {
    handleError("failed to create $format output", $efile);
  }

  my $fh;
  unless($out){
    $fh = \*STDOUT;
  }else{
    unless(open($fh, ">$out")){
      close $fh;
      handleError("Could not open $out for writing : $!");
    }
  }

  if (-r $resfile && open(OUTPUT, "< $resfile")) {
    while (<OUTPUT>) { print $fh $_; }
    close(OUTPUT);
  }
  else {
    print STDERR "$0: could not read result $resfile : $!\n";
  }
  close $fh;

  ($res, $msg) = $iprscan->cleanup($h_chunk);
  handleError($msg, $efile) unless $res;
  
  handleError(undef, $efile) if -e $efile && -s $efile;

  exit;
}
elsif($cmd eq 'check'){

  print STDERR "\nWe are in the check loop:\n\n" if($verbose);

  my $rfile = $iprscan->getConfigValue('toolraw');
  $iprscan->errorExit("iprscan toolraw not configured", undef, 1) unless $rfile;
  my $rfiletmp = $rfile.".resub";

  my $efile = $iprscan->getConfigValue('toolerrors');
  $iprscan->errorExit("iprscan toolerrors not configured", undef, 1) unless $efile;

  my $xfile = $iprscan->getConfigValue('toolxml');
  $iprscan->errorExit("iprscan toolxml not configured", $efile, 1) unless $xfile;

  my $pfile = $iprscan->getConfigValue('toolparams');
  $iprscan->errorExit("iprscan toolparams not configured", $efile, 1) unless $pfile;

  my $workdir = $iprscan->getConfigValue('workdir');
  $iprscan->errorExit("iprscan workdir not configured", $efile, 1) unless $pfile;

  #read again the params file to get some informations...
  $params = { };
  print STDERR "Reading Params file [$pfile] ... " if($verbose);
  ($res, $msg) = $iprscan->readParamFile($pfile, $params);
  handleError("$tname: Problem reading params file : $msg") unless($res);
  print STDERR "DONE\n" if($verbose);
  $iprscan->setParams($params);

  my $href = { };
  my $alldone = 1; #be optimistic
  my $jobs = { };

  ############################   Start of the critical loop!!!  ########################
  #Here we need to check that all the jobs are finished!

  foreach my $chunk (sort {$a <=> $b} keys %$h_chunk){

    print STDERR "Loop over chunk $chunk :\n" if($verbose);

    foreach my $ref (@{$h_chunk->{$chunk}->{appl}}){

      my $jobid = $ref->[1];
      my $appl  = $ref->[0];
      #FIXME, jobs must be removed to avoid problem. So use the my $jobs = $h_chunk->{$chunk}->{jobs} instead;

      print STDERR "\tJOBID: $jobid\n\tAPPL: $appl\n" if($verbose);

      $href->{jobid} = $jobid;
      $href->{session} = $iprscan->getParam('jobid');
      $href->{chunk} = "chunk_$chunk";

      my ($mess, $tool);
      my $tname = $iprscan->mapApplNameToToolName($appl);

      print STDERR "\t[$chunk] Creating Tool for [$tname] ... " if($verbose);
      ($res, $tool)  = $iprscan->createTool($tname, $href);
      $iprscan->errorExit($tool, $efile, 1) unless($res);
      print STDERR "\tDONE\n" if($verbose);

      my $toolerrors = $tool->getConfigValue('toolerrors');
      $iprscan->errorExit("$tname toolerrors not configured", $efile, 1) unless $toolerrors;

      my $toolexitcode = $tool->getConfigValue('toolexitcode');
      $iprscan->errorExit("$tname toolexitcode not configured", $efile, 1) unless $toolexitcode;

      my $tooloutput = $tool->getConfigValue('tooloutput');
      $iprscan->errorExit("$tname tooloutput not configured", $efile, 1) unless $tooloutput;

      print STDERR "\t[$chunk] Adding chunk ... " if($verbose);
      ($res, $mess) = $iprscan->addChunk($chunk, \$toolerrors, \$toolexitcode, \$tooloutput, \$rfiletmp, \$workdir);
      $iprscan->errorExit($msg, undef, 1) unless$res;
      print STDERR "DONE\n" if($verbose);

      print STDERR "\n\t[$chunk] Resulting files are:\n" if($verbose);
      print STDERR "\t\tERR:$toolerrors\n\t\tEXCDE:$toolexitcode\n\t\tOUT:$tooloutput\n\n" if($verbose);

      print STDERR "$rfiletmp not here\n" unless(-f $rfiletmp);

      $h_chunk->{$chunk}->{jobs}->{$jobid} = { 'errors' => $toolerrors, 'exitcode' => $toolexitcode, 'output' => $tooloutput };

      if(-f $toolerrors && -s $toolerrors > 0){
	print STDERR "\t[$chunk] Error file here and > 0\n" if($verbose);
	if(-f $tooloutput && -s $tooloutput == 0){
	  print STDERR "\t[$chunk] Output file here and == 0\n" if($verbose);
	  my $toolreport = "$workdir/$jobid.report";
	  print STDERR "\t[$chunk] Appl: $tname ToolReport: $toolreport\n" if($verbose);
	  #Then we consider the jobs failed again. Wrong though?
	  unless(-f $toolreport){
	    print STDERR "\t[$chunk] Creating report for application $tname ... " if($verbose);
	    ($res, $mess) = $iprscan->createFailedReport($jobid, $h_chunk, $toolreport);
	    $iprscan->errorExit($msg, $efile, 1) unless($res);
	    print STDERR "DONE\n" if($verbose);
	  }else{
	    print STDERR "[$chunk] File [$toolreport] laready there.\n" if($verbose);
	  }
	}
      }

      next if(-f $toolexitcode);
      print STDERR "\t[$chunk] ExitCode not here\n" if($verbose);
      $alldone = 0;

    } #end for loop

  }   #end foreach my $chunk loop


  if($alldone){

    #Then we can remove the root report file.
    unless(unlink($iprscan->getConfigValue('toolreport'))){
      $iprscan->errorExit("Could not remove Root Report file : $!", undef, 1) unless $res;
    }

    print STDERR "\n** All jobs DONE! **\n" if($verbose);
    #Need to parse results now and create new raw results...
    print STDERR "Launching redoResults ... " if($verbose);
    ($res, $msg) = redoResults($iprscan, $h_chunk, $rfile);
    $iprscan->errorExit($msg, undef, 1) unless $res;
    print STDERR "DONE\n" if($verbose);

    #We have created all the xml and txt files for each chunk.
    #Now we redirect the user to the page. By default we redirect him/her
    # to the first chunk.
    ($res, $msg) = $iprscan->addChunk('1', \$xfile);
    $iprscan->errorExit($msg, undef, 1) unless($res);

    print header(-type => 'text/html');
    if ($iprscan->getParam('view') eq 'picture') {
      ($res, $msg) = $iprscan->createResultPicture($xfile);
    }
    else {
      ($res, $msg) = $iprscan->createResultTable($xfile);
    }
    $iprscan->errorExit($msg, undef, 1) unless $res;

    ($res, $msg) = Dispatcher::Queue->cleanup('iprscan', $iprscan->getConfigValue('jobid'));
    $iprscan->errorExit($msg, undef, 1) unless($res);

    unlink($efile) unless -s $efile > 0;

  } #end if($alldone)
  elsif (-f $efile && -s $efile > 0) {
    $iprscan->errorExit(undef, $efile, 1);
  }
  else {
    print header(-type => 'text/html');
    my $url = $iprscan->getConfigValue('resuburl');
    $url .= '1';
    ($res, $msg) = $iprscan->createPollPage($url);
    $iprscan->errorExit($msg, undef, 1) unless $res;
    exit;
  }
}else{
  my $url = $iprscan->getConfigValue('submiturl');
  print redirect(-uri => $url);
  exit;
}


######################################
###           SUBROUTINES          ###
######################################

=head2 redoResults

 Description: Parse results of the remade failed jobs and concatenate them with old results.
 Arguments:   $iprscan   Reference to a Dispatcher::Tool::InterProScan object.
              $h_chunk   Reference to a hash table.
              $efile     Path to the error file.
              $rfile     Path to the raw file.
 Returns:     1, ''      On success
              0, message On error(s).

=cut

sub redoResults {
  
  my($iprscan, $h_chunk, $rfile) = @_;
  
  my $defaults = { };
  my $params = { };
  my($appl, $tname, $tool, $oclose, $ofh, $raw);
  my($res, $msg);

  print STDERR "\n\n=====================================\n" if($verbose > 1);
  print STDERR "\nEntering 'redoResults' subroutine.\n" if($verbose > 1);
  
  my $seqs = { };
  my $toolnocrc = $iprscan->getConfigValue('toolnocrc');
  my $workdir = $iprscan->getConfigValue('workdir');

######################### Start of critical loop!! ########################

  foreach my $chunk (sort {$a <=> $b} keys %$h_chunk){

    ($res, $msg) = $iprscan->addChunk($chunk, \$toolnocrc, \$rfile, \$workdir);
    return(0, $msg) unless $res;

    ($res, $oclose, $ofh) = $iprscan->getWriteHandle($rfile);
    return(0, $ofh) unless $res;


    #We need to reload info (id, len, crc64) about sequences. We do it from params file.
    $params = $iprscan->getParams();
    print STDERR "Rebuilding seqs hash table for chunk $chunk ... " if($verbose > 1);

    foreach my $key (grep { /^seq\.$chunk\.(\d+)\.id$/ } keys %$params){
	my $id = $params->{$key};
	my $num = ($key =~ /$chunk\.(\d+)\./);
	$seqs->{$id} = { } unless $seqs->{$id};
	$seqs->{$id}->{len} = $iprscan->getParam("seq.$chunk.$num.len");
	return(0, "Unable to find seq.$chunk.$num.len in params") unless $seqs->{$id}->{len};
	$seqs->{$id}->{crc64} = $iprscan->getParam("seq.$chunk.$num.crc64");
	return(0, "Unable to find seq.$chunk.$num.crc64 in params") unless $seqs->{$id}->{crc64};
    }
    print STDERR "\tseqs hash table empty!! Problems?? " if($verbose  > 1 && !keys(%$seqs));
    print STDERR "DONE\n" if($verbose > 1);

    # We did not copy toolnocrc before so we don't need to remove it.
    #unlink("$toolnocrc.tmp");

    my $jobs = $h_chunk->{$chunk}->{jobs};

    foreach my $jobid (keys %$jobs){
      print STDERR "\t[$chunk] Jobid : $jobid\n" if($verbose > 1);
      $appl = $iprscan->mapJobIDToApplName($jobid);
      return(0, "could not map application name from $jobid") unless $appl;

      $tname = $iprscan->mapApplNameToToolName($appl);
      return(0, "$appl not supported") unless $tname;

      if ($jobid && $jobid =~ /^(\w+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d))-\w+-cnk(\d+)/) {
	# set date and time because they may be used in configuration (paths and URLs)
	$defaults->{YYYY} = $2;
	$defaults->{MM} = sprintf("%.2d", $3);
	$defaults->{DD} = sprintf("%.2d", $4);
	$defaults->{hh} = sprintf("%.2d", $5);
	$defaults->{mm} = sprintf("%.2d", $6);
	$defaults->{ss} = sprintf("%.2d", $7);
	$defaults->{session} = $1;
	$defaults->{chunk}   = "chunk_$9";
      }
      else{
	return(0, "Could not determine jobid from $jobid") unless $jobid;
      }

      $defaults->{jobid}    = $jobid;
      $defaults->{srchtype} = $iprscan->getParam('srchtype');

      print STDERR "\t[$chunk] Creating tool for $tname ... " if($verbose > 1);
      ($res, $tool) = $iprscan->createTool($tname, $defaults);
      return(0, $tool) unless $res;
      print STDERR "DONE\n" if($verbose > 1);
      my $tooloutput = $tool->getConfigValue('tooloutput');
    
      next unless -e $tooloutput;  # CRC matched for this sequence or no hit have been found
      my $result = { };
    
      #EQ Maybe the best place to index the results at the same time than parse them...
      my $inx =  { } unless($iprscan->getParam('submission' eq 'cli'));

      print STDERR "\t[$chunk] Parsing results ... " if($verbose  > 1);
      ($res, $msg) = $tool->parseResult($result, $tooloutput, $inx);
      return(0, $msg) unless $res;
      print STDERR "DONE\n" if($verbose  > 1);
    
      unless($iprscan->getParam('submission') eq 'cli'){
	my $index;
	($res, $index) = Index->new($tooloutput);
	return(0, "$appl : $index") unless($res);
	
	($res, $msg) = $index->indexOut($inx);
	return(0,"$appl : $msg") unless($res);; 
      }

      if($appl eq 'hmmsmart'){
	my $smd = $tool->getConfigValue('smartdesc');
	my $smt = $tool->getConfigValue('smartthresholds');
	
	if(-e $smd && -e $smt){
	  print STDERR "\t[$chunk] Running smart filtering ... " if($verbose > 1);
	  foreach my $hit (sort { $a <=> $b } grep { /\d+/ } keys %$result){
	    ($res, $msg) = $tool->doSmartFilter($result->{$hit});
	    delete($result->{$hit}) unless($res);
	  }
	  print STDERR "DONE\n" if($verbose > 1)
	}
      }
      print STDERR "\t[$chunk] Getting Raw results ... " if($verbose > 1);
      ($res, $raw) = $tool->getRawResult($result, $seqs, $iprscan, $iprscan->getParam('iprlookup'), $iprscan->getParam('goterms'));
      return(0, "$appl : $raw") unless($res);
      print STDERR "DONE\n" if($verbose > 1);
      print $ofh $raw;

    } #end foreach my keys %jobs
    close $ofh if $oclose;
    #We now need to concatenate the new raw file with the old one!
    
    print STDERR "\t[$chunk] Concatenate raw file ... " if($verbose > 1);
    if(-f "$rfile.resub"){
      if(system("cat $rfile >> $rfile.resub")){
	return(0, "Unable to concatenate $rfile : $!");
      }
      unless(rename("$rfile.resub", $rfile)){
	return(0, "Unable to rename $rfile : $!");
      }
    }
    print STDERR "DONE\n" if($verbose > 1);

    #Now we can create xml output for each chunk (if we are under CGI request only)
    if($html){
      my $xmlcmd = $iprscan->getConfigValue('iprscan.xml.converter');
      return(0, "$tname iprscan.xml.converter not configured") unless $xmlcmd;
      
      my $format = $iprscan->getParam('format');
      
      return(0, "$tname: no format!") unless($format);
      $xmlcmd =~ s/\%format/$format/;
      $xmlcmd =~ s/\%toolraw/$rfile/;

      #We create XML output
      #####################
      my $xfile = $iprscan->getConfigValue('toolxml');
      my $txtfile = $iprscan->getConfigValue('tooltxt');
      my $report = $iprscan->getConfigValue('toolreport');
      my $efile = $iprscan->getConfigValue('toolerrors');

      ($res, $msg) = $iprscan->addChunk($chunk, \$xfile, \$txtfile, \$report, \$efile);
      return(0, $msg) unless($res);
      print STDERR "\t[$chunk] Converting raw to xml ... " if($verbose > 1);
      system("$xmlcmd > $xfile.tmp 2> $efile");
      return(0, "failed to create XML output") if -s $efile > 0;
      print STDERR "DONE\n" if($verbose > 1);

      print STDERR "\t[$chunk] Renaming xmltmp ... " if($verbose > 1);
      unless (rename("$xfile.tmp", $xfile)) {
	return(0, "failed to rename $xfile.tmp: $!");
      }
      print STDERR "DONE\n" if($verbose > 1);
      unlink($efile) if(-s $file == 0);

      #We create now TXT output
      #########################

      my $txtcmd = $iprscan->getConfigValue('iprscan.xml.converter');

      $txtcmd =~ s/\%format/txt/;
      $txtcmd =~ s/\%toolraw/$rfile/;

      print STDERR "\t[$chunk] Converting raw to txt ... " if($verbose > 1);
      system("$txtcmd > $txtfile.tmp 2> $efile");
      return(0, "failed to create TXT output") if -s $efile > 0;
      print STDERR "DONE\n" if($verbose > 1);

      print STDERR "\t[$chunk] Renaming xmltxt ... " if($verbose > 1);
      unless (rename("$txtfile.tmp", $txtfile)) {
	return(0, "failed to rename $txtfile.tmp: $!");
      }
      print STDERR "DONE\n" if($verbose > 1);

      unlink($efile) if(-s $file == 0);

      print STDERR "\t[$chunk] Merging all the report files ... " if($verbose);
      #We also need to concatenate all the eventual appl.report file created during the run.
      if(system("find $workdir -name \\*.report -exec cat {} > $report.tmp \\; -exec rm  {} \\;")){
	return(0, "Could not concatenate report file to [$report.tmp] : $!");
      }
      print STDERR "DONE\n" if($verbose > 1);

      print STDERR "\t[$chunk] Renaming report.tmp ... " if($verbose > 1);
      unless(rename("$report.tmp", $report)){
	return(0, "Could not rename [$report.tmp] to [$report] : $!");
      }
      print STDERR "DONE\n" if($verbose > 1);

      unlink($report) if(-s $report == 0);

    } #end if($html)

    #We clean each chunk
    print STDERR "\t[$chunk] Cleaning up ... " if($verbose > 1);
    ($res, $msg) = $iprscan->cleanup($h_chunk, $chunk);
    $iprscan->errorExit($msg, undef, 1) unless $res;
    print STDERR "DONE\n\n" if($verbose > 1);

 } #end foreach my $chunk

  print STDERR "Leaving 'redoResults' subroutine\n" if($verbose > 1);
  print STDERR "=================================\n\n" if($verbose > 1);
  return(1, '');
}


=head2 readReportFile

 Description: Read the Main report file in the workdir and put info into a hash table
 Arguments:   $iprscan  Reference to a Dispatcher::Tool::InterProScan object
              $h_chunk Reference to a hash table.
 Returns:     1, Filled hash table
              0, msg on error(s)

=cut

  sub readReportFile ($$){
    my($iprscan, $h_chunk, $report) = @_;

    return(0, "No reference to InterProScan object given") unless($iprscan && $iprscan->isa("Dispatcher::Tool::InterProScan"));
    return(0, "Need a reference to a hash table") unless($h_chunk && ref($h_chunk) eq 'HASH');
    return(0, "REPORT file [$report] not found : $!") unless(-f $report);

    unless(open(REP,$report)){
      close REP;
      return(0, "Could not open [$report] : $!");
    }

    my($cnknum, $jobid) = ('', '');

    while(<REP>){
      chomp;
      if($_ =~ /^Chunk:\s+(\d+)/i){
	$h_chunk->{$1} = { } unless(exists($h_chunk->{$1}));
	$cnknum = $1;
      }elsif($_ =~ /^Jobid:\s+(.+)/i){
	$jobid = $1;
      }elsif($_ =~ /^Application:\s+(\S+)/){
	my $appl = $1;
	if($jobid =~ /$appl/){
	  $h_chunk->{$cnknum}->{appl} = [ ] unless(exists($h_chunk->{$cnknum}->{appl}) && ref($h_chunk->{$cnknum}->{appl}) eq 'ARRAY');
	  push(@{$h_chunk->{$cnknum}->{appl}}, [$appl, $jobid]);
	  $jobid;
	  $appl;
	}
      }
    }
    close REP;

    return(1, $h_chunk);
  }


=head2 handleError

 Description: Handles CLI error and exits.
 Arguments:   -
 Returns:     -

=cut

sub handleError(;$$) {
  my ($emsg, $efile) = @_;
  
  print STDERR "$0: $emsg\n" if $emsg;
  exit unless -e $efile;
  
  unless (open(EFILE, "< $efile")) {
    print STDERR "$0: could not read file $efile: $!\n";
    exit;
  }
  
  while (<EFILE>) { print STDERR $_; }
  close(EFILE);
  exit;
}

=head2 usage

 Description: Prints usage
 Arguments:   -
 Returns:     -

=cut

sub usage {
  
  print STDERR "Usage : $0 -r <reportfile> -h -v [0,1,2]\n\n";
  print STDERR "        -r  Report file where the errors are stored\n";
  print STDERR "            ( e.g. /path/to/iprscan/tmp/20040302/iprscan-20040302-12355481/iprscan-20040302-12335481.report).\n";
  print STDERR "        -h  Displays this help on STDERR and exit.\n";
  print STDERR "        -v  Display messages during run on STDERR.\n";
  print STDERR "            0 no infos at all (like without -v option).\n";
  print STDERR "            1 only prints main actions.\n";
  print STDERR "            2 prints all what the script is doing.\n";
  print STDERR "\n";
  
}
