#!/ebi/extserv/bin/perl/bin/perl

=head1 NAME

meter.pl - tool to know percentage of jobs done.

=head1 SYNOPSIS

  # examples here
  ./meter.pl <session_dir|dir> -h (help)


=head1 DESCRIPTION

=head1 VERSIONS

  $Id: meter.pl,v 1.1.1.1 2005/08/18 13:18:25 hunter Exp $

  Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

BEGIN {
$ENV{IPRSCAN_HOME} ='/scratch/tmp/iprscan';
}

use strict;
use Getopt::Long;
use lib "$ENV{IPRSCAN_HOME}/lib";
use Dispatcher::Tool;

my $help;
my $jobid;
my $wdir;
my $iprscan;
my ($res, $msg);

GetOptions('j=s' => \$jobid, 'h'   => \$help);

if($help){
  usage();
}

unless($jobid){

    $wdir = shift;
    usage("No directory given") unless($wdir);
    die "$wdir is not a directory : $!" unless(-d $wdir);

}else{
    unless($jobid =~ /^[A-Za-z]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d)/){
	die "Jobid malformed!";
    }
    my $defaults = { };
    $defaults->{YYYY} = $1;
    $defaults->{MM} = sprintf("%.2d", $2);
    $defaults->{DD} = sprintf("%.2d", $3);
    $defaults->{jobid} = $jobid;

    $iprscan = Dispatcher::Tool->new();
    $iprscan->setToolName('iprscan');
    ($res, $msg) = $iprscan->parseConfig(1, $defaults);
    die $msg unless $res;
    $wdir = $iprscan->getConfigValue('workdir');
    die "$wdir is not a directory : $!" unless(-d $wdir);
}

my ($Ncnk,$Ncnk_done) = (0,0);

unless(opendir(DIR, "$wdir")){
  closedir DIR;
  usage($!);
}

foreach my $d (readdir(DIR)) {
  next unless ($d =~ /^chunk_\d+/);
  $Ncnk++;
  $Ncnk_done++ if (-f "$wdir/$d/merged.raw"); 
}
closedir(DIR);

usage("Unable to find any 'chunk' directory in '$wdir'") unless($Ncnk);

my $percentage =  sprintf ("%.0f",$Ncnk_done*100/$Ncnk);

print "\nPercentage of chunk(s) done : $percentage% ($Ncnk_done over $Ncnk)\n\n";

exit;

sub usage {
  my ($mess) = @_;
  print "ERROR: $mess\n" if($mess);
  print "Usage: $0 [<session-dir>|-j <jobid>] -h <help>\n";
  print "                  -j jobid. Find automatically the session direcotry.\n";
  print "                  -h Displays this help and exits.\n";
  exit;
}
