#!/ebi/extserv/bin/perl/bin/perl

# *************************************************************
# Purpose: 	Filter a fasta database to keep only the N-term
# 		part of the sequences as required for some
#		programs such as SignalP
# 
# 		This script takes a protein fasta database on 
#		STDIN and a cutoff number $NBAA_SIGNALP on the 
#		command line. It return the database on STDOUT 
#		keeping only the first $NBAA_SIGNALP amino 
#		acids (N-term)
#
# IMPORTANT:	You MUST filter your fasta database with this 
#		script before running SignalP (suggested value 
# 		by SignalP authors for $NBAA_SIGNALP is 100)
#
# $Source: /ebi/cvs/interpro/InterProScan/v4/iprscan/bin/trim_in.pl,v $
# $Revision: 1.1.1.1 $
# $Date: 2005/08/18 13:18:25 $
# $Author: hunter $
#
# USAGE:
# *************************************************************

use strict;
use Getopt::Long;

my $usage = "$0 -n <number_of_aa> -i <input_file>";
my ($NBAA_SIGNALP);
my ($input);

unless(GetOptions('n=i' => \$NBAA_SIGNALP,
		  'i=s' => \$input
		 )){
  print STDERR "\n  $usage \n\n";
	exit(1);
}

$NBAA_SIGNALP ||= '100';

unless($input) { print STDERR "Need a file to analyze.\n$usage\n"; exit(1); }
unless(-f $input) { print STDERR "$input does not exist : $!\n"; exit(1); }
unless(open(F, "<$input")){ print STDERR "Could not open $input : $!\n"; exit(1); }

my ($cpt_aa) = (0);	# number of printed aa for a given protein sequence
my $line;

while ( defined($line=<F>) ) 
{
	chomp($line);
	if ($line =~ /^\>/)	# Print the comment line with the sequence name "as it is"
	{
		print STDOUT "$line\n";
		$cpt_aa = 0;
	}
	elsif ( $cpt_aa < $NBAA_SIGNALP)	# Print the aa sequence contained in the line up to $NBAA_SIGNALP aa
	{
		$line =~ s/\s+//g;
		my @F = split("", $line);	# Array of aa for the current line aa sequence
		my $i = 0;			# Index number for the array @F
		while ( ($i <= $#F) && ($cpt_aa < $NBAA_SIGNALP) )
		{
			print STDOUT "$F[$i]";
			$cpt_aa ++;
			$i ++;
		}
		print STDOUT "\n";
	}
}
close F;
