#!/ebi/extserv/bin/perl/bin/perl

=head1 NAME

 wget.pl - Retrieve entries from indexed file or display status of indexed files

=cut

=head1 SYNOPSIS

=over 5

=item Command line only

=item wget.pl -cli [-s] [-f <indexed file>] 

=item         [-t <toolname>] [-e <entry>] [-d <db name>] 

=item         [-o <output file>] [-i] [-n] [-h]

=cut

=head1 DESCRIPTION

 Used to query interpro.xml datafile and check the status of the indexed datafiles.

=cut

=head1 VERSIONS

 $Id: wget.pl,v 1.2 2005/10/17 16:05:11 hunter Exp $

 Copyright (c) European Bioinformatics Institute 2002

=cut

=head1 AUTHORS / ACKNOWLEDGEMENTS

 Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut


BEGIN {
$ENV{IPRSCAN_HOME}='/scratch/tmp/iprscan';
  $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
}

use CGI qw(:standard :html -no_debug);
use CGI::Carp qw(fatalsToBrowser carpout);
use English;
use Getopt::Long;
use Term::ANSIColor;
use Date::Manip qw(DateCalc UnixDate);
use XML::Quote;
use strict;

#If you keep this script in its original place and don't move the lib directory or create a symbolic link.
use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool::InterProScan;


$OUTPUT_AUTOFLUSH = 1;

my $module = { 'InterPro'         => {'pack' => 'Index::InterPro',
				      'file' => 'interpro.xml',
				      'db'   => 'InterPro'},
	       'IprMatches'       => {'pack' => 'Index::IprMatches',
				      'file' => 'match.xml',
				      'db'   => 'IprMatches'},
	       'SmartDesc'        => {'pack' => 'Index::SmartDesc',
				      'file' => 'smart.desc',
				      'db'   => 'Smart'},
	       'SmartThresholds'  => {'pack' => 'Index::SmartThresholds',
				      'file' => 'smart.thresholds',
				      'db'   => 'Smart'},
	       'Pfam'             => {'pack' => 'Index::Hmm',
				      'file' => 'Pfam',
				      'db'   => 'Pfam'},
	       'Superfamily'      => {'pack' => 'Index::Hmm',
				      'file' => 'superfamily.hmm',
				      'db'   => 'SUPERFAMILY'},
	       'PIR'              => {'pack' => 'Index::Hmm',
				      'file' => 'sf_hmm',
				      'db'   => 'PIR'},
	       'Tigr'             => {'pack' => 'Index::Hmm',
				      'file' => 'TIGRFAMs_HMM.LIB',
				      'db'   => 'TIGR'},
	       'Smart'            => {'pack' => 'Index::Hmm',
				      'file' => 'smart.HMMs',
				      'db'   => 'Smart'},
	       'Prints'           => {'pack' => 'Index::Prints',
				      'file' => 'prints.pval',
				      'db'   => 'PRINTS'},
#	       'Gene3D'           => {'pack' => 'Index::Hmm',
#				      'file' => 'Gene3D.hmm',
#				      'db'   => 'Gene3D'},
	       'Pfam-Clan'        => {'pack' => 'Index::PfamClan',
				      'file' => 'Pfam-C',
				      'db'   => 'Pfam-Clan'},
	       'Pfam-A'           => {'pack' => 'Index::PfamA',
				      'file' => 'Pfam-A.seed',
				      'db'   => 'Pfam-A'}
	     };

#Used to convert InterPro, ProDom, ... entry to a link to the database home page.
my $entryurl = { 
		'PRODOM'     => 'http://prodes.toulouse.inra.fr/prodom/current/cgi-bin/request.pl?question=DBEN&query=',
		'PRINTS'     => 'http://bioinf.man.ac.uk/cgi-bin/dbbrowser/sprint/searchprintss.cgi?display_opts=Prints&category=None&queryform=false&prints_accn=',
		'PROSITE'    => 'http://www.expasy.ch/cgi-bin/prosite-search-ac?',
		'PROFILE'    => 'http://www.isrec.isb-sib.ch/cgi-bin/get_pstprf?',
		'SMART'      => 'http://smart.embl-heidelberg.de/smart/do_annotation.pl?BLAST=DUMMY&ACC=',
		'PFAM'       => 'http://www.sanger.ac.uk/cgi-bin/Pfam/getacc?',
		'TIGRFAMs'   => 'http://www.tigr.org/tigr-scripts/CMR2/hmm_report.spl?user=access&password=access&acc=',
		'PROSTE'     => 'http://www.expasy.ch/cgi-bin/prosite-search-ac?',
		'PIRSF'      => 'http://pir.georgetown.edu/cgi-bin/ipcSF?id=',
		'SSF'        => 'http://supfam.org/SUPERFAMILY/cgi-bin/scop.cgi?ipid=',
		'IPR'        => 'http://www.ebi.ac.uk/interpro/IEntry?ac=',
		'PREFILE'    => 'http://www.isrec.isb-sib.ch/cgi-bin/get_pstprf?',
		'GO'         => 'http://www.ebi.ac.uk/ego/DisplayGoTerm?id=',
		'SCOP'       => 'http://supfam.org/SUPERFAMILY/cgi-bin/scop.cgi?ipid=',
		'PUBMED'     => 'http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&dopt=Abstract&list_uids=',
		'PDB'        => 'http://www.rcsb.org/pdb/cgi/explore.cgi?pdbId=',
		'IUPHAR'     => 'http://iuphar-db.org/iuphar-rd/query?submit=Search&type=search&sString=',
		'BLOCKS'     => 'http://bioinformatics.weizmann.ac.il/blocks-bin/getblock.sh?',
		'GENE3D'     => 'http://www.biochem.ucl.ac.uk/cgi-bin/grant/gene3d.pl?domainchoice=',
		'CATH'       => 'http://cathwww.biochem.ucl.ac.uk/cgi-bin/cath/GotoCath.pl?cath=',
		'COMe'       => 'http://www.ebi.ac.uk/come/entry?gn=',
		'EC'         => 'http://ca.expasy.org/cgi-bin/nicezyme.pl?',
		'MEROPS'     => 'http://merops.sanger.ac.uk/cgi-bin/strusearch.pl?Clan_Family=',
		'CAZY'       => 'http://afmb.cnrs-mrs.fr/CAZY/query.php?fam=',
		'PDOC'       => 'http://ca.expasy.org/cgi-bin/nicedoc.pl?',
		'PANDIT'     => 'http://www.ebi.ac.uk/goldman-srv/pandit/pandit.cgi?action=browse&fam=',
		'MSDsite'    => 'http://www.ebi.ac.uk/msd-srv/msdsite/barChartPattern?pattern1=',
		'PROSITEDOC' => 'http://ca.expasy.org/cgi-bin/nicedoc.pl?',
		'PANTHER'    => 'https://panther.appliedbiosystems.com/panther/family.do?clsAccession='
	       };

my ($file, $tool, $iprscan, $entry, $out, $dbname, $index, $insens, $nomore) = (undef, undef, undef, undef, undef, undef, undef, undef);
my($res, $msg, $help, $query);
my $html = 1;


if($ARGV[0] eq '-cli'){
    $html = 0;
    shift;
    unless(GetOptions(
		      's'       => \$index,
		      'f=s'     => \$file,
		      't=s'     => \$tool,
		      'e=s'     => \$entry,
		      'd=s'     => \$dbname,
		      'o=s'     => \$out,
		      'i'       => \$insens,
		      'n'       => \$nomore,
                      'h'       => \$help
		     )){
	usage();
    }

    if($help){
	usage();
    }
}else{
    $query = new CGI();

    $index  = $query->param('index')  || 0;
    $dbname = $query->param('dbname') || undef;
    $entry  = $query->param('query')  || $query->param('entry') || 0;
    $file   = $query->param('file')   || undef;
    $insens = $query->param('case')   || 0;
    $tool   = $query->param('tool')   || undef;
}

my $ofh;
if($out){
    weberror("Could not open $out : $!", $html) unless(open($ofh, ">$out"));
}
else{
    $ofh = \*STDOUT;
}


#If you want to use it, move it down after creation of InterProScan object instanciation.
#We want to access the tool box
#if($query->param('toolbox')){
#    if($query->param('getcrc')){
#      unless ($query->param('sequence') || $query->param('upfile')){
#	weberror('no sequence given', 'toolbox');
#	exit;
#      }
#      my($res, $msg) = searchforCRC($iprscan);
#      weberror($msg, 'toolbox') unless($res);
#      exit;
#    }elsif($query->param('viewEntry')){
#      my @num = $query->param('seqn');
#      my @bak = $query->param('info');
#      my($res, $msg) = searchforCRC($iprscan, \@num, \@bak);
#      weberror($msg, 'toolbox') unless($res);
#      exit;
#    }
#    else{
#      my($res, $msg) = displayToolBox($iprscan);
#      weberror($msg, 'toolbox') unless($res);
#      exit;
#    }
#}

$iprscan = Dispatcher::Tool::InterProScan->new();
($res, $msg) = $iprscan->setToolName('iprscan');
weberror($msg, $html) unless $res;
($res, $msg) = $iprscan->parseConfig(1);
weberror($msg, $html) unless$res;

my $HOME = $iprscan->getConfigValue('submiturl');
my $TOOLBOX = $iprscan->getConfigValue('wgeturl.index');


#We display status of each indexed file
if($index){
  my ($res, $msg)  = displayInx($iprscan, $html);
  weberror($msg, $html) unless $res;
  exit;
}

#We do a request in the InterPro database
#Only available for the web interface.
if($html){
    if($query->param('queryForm') || !$query->param()){
	my ($res, $msg) = displayQueryForm($iprscan);
	weberror($msg, $html) unless $res;
	exit;
    }
}

if($dbname && $entry){
  my ($res, $msg) = dorequest($iprscan, $dbname, $entry, $insens);
  weberror($msg, $html) unless $res;
  exit;
}


# We are expecting to display an application method result.
unless($file){
  weberror("No file given as parameter", $html);
}

unless($tool){
  weberror("No tool given as parameter", $html);
}

unless($entry){
  weberror("No entry to search, entry field is empty", $html);
}

unless(-f $file){
  weberror("File $file does not exist : $!", $html);
}

#We create new tool, by default InterProScan to access method from Tool object
my($inx, $tname);

#$tool is not InterProScan, so we create the appropriate tool
unless($tool eq "iprscan"){
  $tname = $iprscan->mapApplNameToToolName($tool);
  weberror("Unable to retrieve Tool Name from $tool", $html) unless($tname);
  ($res, $tool) = $iprscan->createTool($tname);
  weberror("$tname Tool creation failed : $msg", $html) unless($res);
  $iprscan = $tool;
  $tool = $tname;
}

$iprscan->setToolName($tool);

#We parse the configuration file of the new tool created to access values from the conf
($res, $msg) = $iprscan->parseConfig(1);
weberror("Parsing configuration failed : $msg", $html) unless($res);

#We create new Index object to retrieve entry
($res, $inx) = new Index($file);
weberror("Failed to create Index object : $inx", $html) unless($res);

($res, $msg) = $inx->getEntry($entry, undef, $iprscan->getConfigValue('recdel'),undef, 1);
weberror("Unable to retrieve entry : $msg", $html) unless($res);

print STDOUT header() if($html);
print getTitle($entry, $html) unless($nomore);

if(ref($msg) eq "ARRAY"){
  $msg = join("\n", @$msg);
}

if($msg eq ""){
  print $ofh "Your query \"$entry\" returned nothing\n";
}else{
  print $ofh "<PRE>\n" if($html);
  print $ofh "$msg\n";
  print $ofh "</PRE>\n" if($html);
}

exit(0);



=head2 displayInx

 Description: Display the status of the indexed flat files (indexed or not).
 Arguments:   $iprscan   InterProScan tool object.
              $html      CGI usage?
 Returns:     1, ''      on success.
              0, message on error

=cut

sub displayInx ($$) {

  my($iprscan, $html) = @_;
  my($res, $msg, $index);

  my $tool = Dispatcher::Tool->new();
  $tool->setToolName('index');

  ($res, $msg) = $tool->parseConfig(1);
  return(0, $msg) unless $res;

  my $imgok    = $iprscan->getConfigValue('imgurl.inx.ok');
  my $imgnotok = $iprscan->getConfigValue('imgurl.inx.notok');
  my $imgcross = $iprscan->getConfigValue('imgurl.inx.cross');
  my $imgico   = $iprscan->getConfigValue('imgurl.icon');
  my $imgiprs  = $iprscan->getConfigValue('imgurl.iprs');
  my $date = `date +"[%a %d %b %Y (%H:%M:%S)]"`;
  chomp($date);

  if($html){
      print STDOUT header();
      print STDOUT "<head>\n<link rel=\"SHORTCUT ICON\" href=\"$imgico\">\n";
      print STDOUT "<title>InterProScan (interhelp\@ebi.ac.uk) : Indexes status</title></head>\n";
      print STDOUT "<body><br><table border=\"0\" width=\"100%\"><tr><td align=\"center\">\n";
      print STDOUT "<table width=\"600\" border=\"1\"><tr><td colspan=\"5\" align=\"center\"><img src=\"$imgiprs\" border=\"0\"></td><tr>\n";
      print STDOUT "<tr><td><table  width=\"600\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n";
      print STDOUT "<tr bgcolor=\"#dddddd\"><td colspan=\"5\" align=\"center\"><b>Indexes status $date</b></td></tr>\n";
      print STDOUT "<tr bgcolor=\"#dddddd\"><td colspan=\"5\" align=\"center\">&nbsp;</td></tr>\n";
      print STDOUT "<tr bgcolor=\"#dddddd\"><td align=\"center\"><b>Database</b></td><td align=\"center\"><b>Datafile</b></td><td align=\"center\"><b>Size (bytes)</b></td>";
      print STDOUT "<td align=\"center\"><b>Last modification time</b></td><td align=\"center\"><b>Status</b></td></tr>";
  }else{
      print $ofh &getBanner("InterProScan : Indexes status $date");
      print $ofh sprintf("%-15s%-20s%-20s%-30s%-15s\n", "Database", "File", "Size (bytes)", "Last modification time", "Status");
      print $ofh sprintf("%-45s\n\n", "=" x 105 );
  }

  my $datadir = "$ENV{IPRSCAN_HOME}/data";

  for(sort keys %$module){
      my $caller = $module->{$_}->{pack};
      my $file   = $module->{$_}->{file};
      my $db     = $module->{$_}->{db};

      next if("$datadir/$file" =~ /smart\.desc|smart\.thresholds/ and !-f "$datadir/$file");

      ($res, $index) = $caller->new("$datadir/$file", 1);

      unless($res){
	  if($html){
	      print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"center\">$db</td><td align=\"center\">$file</td><td align=\"center\" colspan=\"3\"><font color=\"#ff0000\">ERROR!</font></td></tr>\n";
	      print STDOUT "<tr bgcolor=\"#eeeeee\"><td colspan=\"5\" align=\"center\">$index</td></tr>\n";
	  }else{
	      print $ofh sprintf("%-15s\t%-20s%-15s", $db, $file, colored("ERROR: $index ", "red"));
	      print $ofh "\n\n";
	  }
	  next;
      }

      my $info;
      ($res, $msg, $info) = $index->checkIndex();

      my ($date, $secs, $size, $mtime);# = ('&nbsp;', '&nbsp;', '&nbsp;', '&nbsp;');

      if($info){
	  ($size, $mtime) = split(/\|/, $info);
	  $secs = DateCalc("Jan 1, 1970  00:00:00 GMT",$mtime);
	  $date = UnixDate($secs,"%d/%m/%Y [%H:%M:%S]");
      }

      unless($res){
	  $res = '2' unless(-f $index->{inx} && -f $index->{file});
      }

      my $status = $res ? $res eq '2' ? $imgcross : $imgok : $imgnotok;
      my $alt = $res ? $res eq '2' ? 'Index file not found' : 'Index ok' : 'Index out of date';

      if($html){
	  print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"center\">$db</td><td align=\"center\">$file</td>";
	  print STDOUT "<td align=\"center\">$size</td><td align=\"center\">$date</td>\n";
	  print STDOUT "<td align=\"center\"><img src=\"$status\" alt=\"$alt\" border=\"0\"></td></tr>\n";
      }else{
	  my $color = $res ? $res eq '2' ? 'red underline' : 'green dark' : 'yellow underline';
	  print $ofh sprintf("%-15s%-20s%-20s%-30s%-15s", $db, $file, $size, $date, colored ("$alt", $color)), "\n";
      }

  }

  if($html){
	my $imgok    = $iprscan->getConfigValue('imgurl.inx.ok');
	my $imgnotok = $iprscan->getConfigValue('imgurl.inx.notok');
	my $imgcross = $iprscan->getConfigValue('imgurl.inx.cross');
	print STDOUT "</table></td></tr>\n";
	print STDOUT "<tr><td colspan=\"5\" valign=\"middle\"><img src=\"$imgok\" alt=\"Index ok\" border=\"0\">&nbsp<b>Index ok</b><br>\n";
	print STDOUT "<img src=\"$imgnotok\" alt=\"Index out of date\" border=\"0\">&nbsp<b>Index out of date</b><br>\n";
	print STDOUT "<img src=\"$imgcross\" alt=\"Index file not found\" border=\"0\">&nbsp<b>Indexed file not found</b><br></td></tr></table>\n";
	print STDOUT "<tr><td colspan=\"5\" align=\"center\"><form action=\"$HOME\"><input type=\"submit\" value=\"Home Page\"></form></td></tr>\n";
	print STDOUT "</td></tr></table><br>\n";
	print STDOUT "<center><b><u>NOTE:</u></b>&nbsp;You can reindex your datafiles using the script <b>index_data.pl</b> located in your iprscan/bin directory.</center>\n";
	print STDOUT "<center>Type ./index_data.pl -h to know how to use it and get more informations.</center></body>\n";
	print STDOUT "</html>\n";
    }else{
	print $ofh "\n\nNOTE: You can reindex your datafiles using the script index_data.pl located in your iprscan/bin directory.\n";
	print $ofh "\tType ./index_data.pl -h to know how to use it and get more informations.\n\n";
    }
  
  return (1, '');
}

=head2 dorequest

 Description: Perform a request in InterPro datafile based on InterPro accesion number
              or members database ID.
 Arguments:  $tool   InterProScan tool object.
             $dbname Datafile to search (Perl module to use).
             $query  Accession number or ID to search.
             $insens   Case insensitive?
 Returns:    1, ''      on success.
             0, message on error

=cut

sub dorequest($$$;$$$$){
  my ($tool, $dbname, $what, $insens) = @_;

  return(0, "No tool reference given") unless $tool;
  return(0, "No DBname given") unless $dbname;
  return(0, "Nothing to search? Please enter an accession number or an ID.") unless $what;

  #remove all the spaces from the query
  $what = $tool->trim($what);

  my $imgico  = $tool->getConfigValue('imgurl.icon');
  my $imgiprs = $tool->getConfigValue('imgurl.iprs');
  my $datadir = "$ENV{IPRSCAN_HOME}/data";

  my $entry;

  unless(exists($module->{$dbname})){
      weberror("Database [$dbname] is not supported.", $html);
  }

  my $caller = $module->{$dbname}->{pack};

  my($res, $index) = $caller->new("$datadir/$module->{$dbname}->{file}", 1);

  return(0, $index) unless $res;


  ($res, $entry) = $index->getEntry($what, $insens);
  return(0, $entry) unless($res);

  if($html){
      print STDOUT header();
      print STDOUT "<head>\n<link rel=\"SHORTCUT ICON\" href=\"$imgico\">\n";
      print STDOUT "<title>InterProScan (interhelp\@ebi.ac.uk) : $dbname query results for $what</title></head>\n";
      print STDOUT "<body><br><table border=\"0\" width=\"800\">\n";
      print STDOUT "<tr><td align=\"left\"><form action=\"$HOME\" method=\"GET\"><input type=\"submit\" value=\"HOME\"></form></td></tr>\n";
      print STDOUT "<tr><td align=\"center\">\n";
      print STDOUT "<table width=\"800\" border=\"1\" cellspacing=\"0\">\n";
      print STDOUT "<tr><td align=\"center\"><img src=\"$imgiprs\" border=\"0\"></td><tr>\n";
      print STDOUT "<tr bgcolor=\"#dddddd\"><form name=\"search\" action=\"$TOOLBOX\" method=\"GET\">\n";
      print STDOUT "<td align=\"left\" valign=\"middle\">Search&nbsp;<input type=\"text\" name = \"query\" size = \"20\">&nbsp;in&nbsp;";
      print STDOUT "<select name=\"dbname\" width=\"100\">\n";

      for(sort keys %$module){
	  my $datadir = "$ENV{IPRSCAN_HOME}/data";
	  my $file = $module->{$_}->{file};

	  next if("$datadir/$file" =~ /smart\.desc|smart\.thresholds/ and !-f "$datadir/$file");
	  print STDOUT "\t\t<option value=\"$_\">$_</option>\n";
      }

      print STDOUT "</select>\n&nbsp&nbsp;<b>Case&nbsp;insensitive&nbsp;<input type=\"checkbox\" name=\"case\" value=\"1\">\n";
      print STDOUT "&nbsp;&nbsp;<input type=\"submit\" value=\"Search\">&nbsp;&nbsp;<input type=\"reset\" value=\"Reset\">\n";
      print STDOUT "</td></form></tr>\n";
  }else{
      print $ofh &getTitle($what) unless($nomore);
  }

  if(ref($entry) eq 'ARRAY'){
      my $cnt = 0;
      my $nbEntry = $#{$entry}+1;
      my $msg = "Your request \"$what\" returned $nbEntry results from $module->{$dbname}->{file} datafile.";

      if($html){
	  print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"left\"><b>$msg</b></td></tr>\n";
      }else{
	  print $ofh "$msg\n\n" unless($nomore);
      }

      for(@$entry){
	  $cnt++;
	  my $info = $_;
	  my $cntEntry = "Entry $cnt :";

	  if($html){
	      $info = xml_quote($info);
	      $info = getLinkedInfo($info);
	      print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"left\"><b> $cntEntry</b></td></tr>\n";
	      print STDOUT "<tr><td><pre>$info</pre></td></tr>\n";
	  }else{
	      unless($nomore){
		  print $ofh sprintf("%s\n", "=" x length($cntEntry));
		  print $ofh sprintf("%s\n", $cntEntry);
		  print $ofh sprintf("%s\n\n", "=" x length($cntEntry));
	      }
	      print $ofh $info, "\n\n";
	  }
      }
  }
  else{
      if($html){
	  print STDOUT "<tr><td>Your request \"$what\" returned nothing.</td></tr>\n";
      }else{
	  print $ofh "Your request \"$what\" returned nothing.\n\n";
      }
  }

  if($html){
      print STDOUT "</table>\n";
      print STDOUT "<tr><td align=\"left\"><form action=\"$HOME\" method=\"GET\"><input type=\"submit\" value=\"HOME\"></form></td></tr>\n";
      print STDOUT "</td></tr></table>\n";
      print STDOUT "</body></html>\n";
  }

  return(1, '');
}


=head2 weberror
    Description:  Display an error page and exit.
    Arguments:    $msg  message to display.
                  $html CGI usage?
	          $name action page to go back
                  $usage displays usage for command line only.
    Returns:      HTML code.

=cut

sub weberror($$;$$$) {
  my($msg, $html, $name) = @_;

  my $page = '';
  my $usage = 1 unless($html);

  if($name eq 'toolbox'){
    $page = $TOOLBOX;
  }else{
    $page = $HOME;
  }

  if($html){
      print STDOUT header();
      print STDOUT "<head>\n<title>InterProScan (interhelp\@ebi.ac.uk) : ERROR</title></head>\n";
      print STDOUT "<body><table border=\"1\"><tr><td>\n";
      print STDOUT "<table border=\"0\" cellpadding=\"2\" cellspacing=\"0\"><tr bgcolor=\"#dddddd\"><td align=\"center\"><font color=\"#ff0000\"><b>ERROR!</b></td></tr>\n";
      print STDOUT "<tr bgcolor=\"#dddddd\"><td><u><b>Message:</b></u>&nbsp;$msg</td></tr>\n";
      print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"center\"><form action=\"$page\" method=\"GET\">\n";
      print STDOUT "<input type=\"submit\" value=\"Back\"></form></td></tr>\n";
      print STDOUT "</table></td></tr>\n";
      print STDOUT "</table></body></html>\n";
  }else{
      my $mess = "   ERROR : $msg   ";
      my $frame = "*" x (length($mess) + 6);
      print STDERR "\n\t$frame";
      print STDERR sprintf("\n\t*  %s  *", colored("$mess", "red"));
      print STDERR "\n\t$frame\n\n";
      usage() if($usage);
  }
  exit(1);
}


=head2 displayQueryForm

    Description: Display page to query datafiles.
    Arguments:   $iprscan Refrence to a Dispatcher::Tool::InterProScan object
    Returns:     1, '' on success
                 0, message on error

=cut

sub displayQueryForm($) {

  my($tool) = @_;

  return(0, "Need a ref to a Dispatcher::Tool::InterProScan object") unless $tool->isa("Dispatcher::Tool::InterProScan");

  my $iprs = $tool->getConfigValue('imgurl.iprs');
  my $icon = $tool->getConfigValue('imgurl.icon');
  my $wget = $tool->getConfigValue('wgeturl.index');
  my $home = $tool->getConfigValue('submiturl');

  print header();
  print STDOUT "<head>\n<link rel=\"SHORTCUT ICON\" href=\"$icon\">\n";
  print STDOUT "<title>InterProScan (interhelp\@ebi.ac.uk) - Query Form</title></head>\n";
  print STDOUT "<body>\n";
  print STDOUT "<table border=\"0\" width=\"100%\"><tr align=\"center\"><td>\n";
  print STDOUT "<table border=\"0\" cellspacing=\"0\" cellpadding\=\"3\" witdh=\"50%\"><tr><td align=\"center\" valign=\"middle\"><img src=\"$iprs\"></td></tr>\n";
  print STDOUT "<tr bgcolor=\"#dddddd\"><td align=\"center\"><b>You can use this search box to:</b></td></tr>\n";
  print STDOUT "<tr bgcolor=\"#eeeeee\"><td><ul><li>Query <b>InterPro</b> to retrieve entries via <b>accession number, name, type, name of proteins and GO id</b><br>\n";
  print STDOUT "(e.g. IPR000001/Kringle/Active_site/V8_HIS or 0008236)<br>\n";
  print STDOUT "or via associated accession numbers of Gene3D, Pfam, PRINTS, Prosite, ProDom, Smart, SuperFamily, PIRSF and TigerFams signatures.</li>\n";
  print STDOUT "<li>Query <b>IprMatches</b> to retrieves entry of known protein sequences in InterPro using its <b>crc64</b> or its <b>ID</b></li>\n";
  print STDOUT "<li>Query <b>Gene3D, Pfam, TigerFams, SuperFamily, PIRSF or Smart HMMs</b> datafiles to see HMM models via their <b>NAME</b> or <b>ACC</b></li>\n";
  print STDOUT "</ul></td></tr>\n";
  print STDOUT "<tr bgcolor=\"#dddddd\"><td align=\"center\">&nbsp;</td></tr>\n";
  print STDOUT "<tr><form name=\"search\" action=\"$wget\" method=\"GET\">\n";
  print STDOUT "<td align=\"center\" valign=\"middle\">Search&nbsp;<input type=\"text\" name = \"query\" size = \"20\">&nbsp;in&nbsp;";
  print STDOUT "<select name=\"dbname\" width=\"100\">\n";

  for(sort keys %$module){
      my $datadir = "$ENV{IPRSCAN_HOME}/data";
      my $file = $module->{$_}->{file};

      next if("$datadir/$file" =~ /smart\.desc|smart\.thresholds/ and !-f "$datadir/$file");
      print STDOUT "\t\t<option value=\"$_\">$_</option>\n";

  }

  print STDOUT "</select>\n<b>Case&nbsp;insensitive&nbsp;</b><input type=\"checkbox\" name=\"case\" value=\"1\"></td>\n";
  print STDOUT "<tr><td align=\"center\" valign=\"middle\"><input type=\"submit\" value=\"Search\">&nbsp;&nbsp;<input type=\"reset\" value=\"Reset\">\n";
  print STDOUT "</td></form></tr>\n";
  print STDOUT "<tr><td align=\"right\"><form name=\"search\" action=\"$home\" method=\"GET\"><input type=\"submit\" value=\"Home\"></td></tr>\n";
  print STDOUT "</td></tr></table>\n";
  print STDOUT "</body></html>\n";

  return(1,'');

}


=head2 getLinkedInfo

   Description: Method used to create a http link to the member database home page using the entry id.
   Arguments:   $entry  entry id.
   Returns:     The HTML code.

=cut

sub getLinkedInfo ($) {

    my($entry) = @_;

    return '' unless $entry;

    $entry =~ s!(&lt;)([^<>]+)(&gt;)!<font size+=2 color=\"grey\">$1$2$3</font>!g;
    $entry =~ s!((GO):\d+)!<a href=\"$entryurl->{$2}$1\" target=\"newwindow\">$1</a>!g;
    $entry =~ s!((IPR)\d+)!<a href=\"$entryurl->{$2}$1\" target=\"newwindow\">$1</a>!g;
    $entry =~ s!db=&quot;(\w+)&quot; dbkey=&quot;([\w\.]+)&quot;!db=&quot;$1&quot; dbkey=&quot;<a href=\"$entryurl->{$1}$2\" target=\"newwindow\">$2</a>&quot;!g;
    $entry =~ s!id=&quot;([\w\.]+)&quot; name=&quot;([^\s]+)&quot; dbname=&quot;(\w+)&quot;!id=&quot;<a href=\"$entryurl->{$3}$1\" target=\"newwindow\">$1</a>&quot; name=&quot;$2&quot; dbname=&quot;$3&quot;!g;

    return $entry;
  }


=head2 getTitle

  Description: Returns a title for HMTL or TEXT version
  Arguments:   $entry Entry name to be displayed in the title.
               $html Is it used by CGI?
  Returns:     string with the title formatted for HMTL or TEXT output.

=cut

sub getTitle($$){
    my ($entry, $html) = @_;

    if($html){
	return "<head>\n<title>InterProScan (interhelp\@ebi.ac.uk) : Results for query $entry </title></head>\n";
    }else{
	my $banner = "Results for query $entry";
	my $msg = sprintf("\t%s", "=" x (length($banner)+2));

	return sprintf("%s\n\t %s\n%s\n\n", $msg, $banner, $msg);
    }
}

=head2 getBanner

    Description: Returns a banner with a text inside.
    Arguments:   Text to print.
    Returns:     The banner.

=cut

sub getBanner($){
    my $text = shift;

    $text = "InterProScan wget results\n" unless $text;

    my $banner = sprintf("%s", "=" x (length($text) + 2));
    my $msg = sprintf("\n\t%s\n\t %s\n\t%s\n\n", $banner, $text, $banner);

    return $msg;
}

=head2 usage

    Description: Prints usage on the standard output and exit
    Arguments:   $msg An error message to display before the usage.
    Returns:     Usage & help.

=cut

sub usage (;$) {
    my $msg = shift;
    print "\nERROR: $msg\n\n" if($msg);
    print "Usage : $0 [-cli] [-s] [-f <indexed file>] [-t <toolname>] [-e <entry>] [-d <db name>] [-o <output file>] [-i] [-n] [-h]\n\n";
    print "           -cli   Command line usage.\n";
    print "           -s     Displays the status of the indexed files.\n";
    print "           -f     Application's output file. Requires full path and tool name (option -t).\n";
    print "           -t     Tool name, Application name. Requires option -f\n";
    print "           -e     Entry, query to search.\n";
    print "           -d     Database name to use when requesting indexed data files.\n";
    print "           -o     Output file where to write result.\n";
    print "           -i     Search using case insensitive.\n";
    print "           -n     Do not print extra messages on the output. (Default on).\n";
    print "           -h     Help, displays this help and exit.\n\n";
    print "Databases supported for index checking and requests are:\n\n";
    print "- ", join("\n- ", sort keys %$module), "\n\n";
    print "Examples:\n";
    print "- Search InterPro entry for IPR000023\n";
    print "\t./wget.pl -cli -d InterPro -e IPR000023\n";
    print "- Show me status of my indexed files\n";
    print "\t./wget.pl -cli -s\n";
    print "- Search word 'kringle' in InterPro file, case insensitive\n";
    print "\t./wget.pl -cli -d InterPro -e kringle -i\n";
    print "- Show me the original Pfam results from my last run for my sequence name Q9RHD9.\n";
    print "\t./wget.pl -cli -t hmmpfam -f <path/to/file>/chunk_X/iprscan-XXXXXXXX-XXXXXXXX-hmmpfam-cnkX -e Q9RHD9\n";
    exit;
}


#=head2 displayToolBox
#
#	   Description: Display Tool Box page to access tools.
#	   Arguments:   $iprscan Reference to a Dispatcher::Tool::InterProScan object
#	   Retruns:     1, '' on success
#	                0, message on error
#
#=cut
#
#sub displayToolBox {
#
#    my ($tool) = @_;
#    return(0, "no reference to tool given") unless($tool && $tool->isa("Dispatcher::Tool::InterProScan"));
#    
#    my $iprs = $tool->getConfigValue('imgurl.iprs');
#    my $icon = $tool->getConfigValue('imgurl.icon');
#    my $toolbox = $tool->getConfigValue('imgurl.toolbox');
#    my $wget = $tool->getConfigValue('wgeturl.index');
#
#    print STDOUT header();
#    print STDOUT "<head>\n<link rel=\"SHORTCUT ICON\" href=\"$icon\">\n";
#    print STDOUT "<title>InterProScan (interhelp\@ebi.ac.uk) - Mini ToolBox</title></head>\n";
#    print STDOUT "<body>\n";
#    print STDOUT "<table border=\"0\" width=\"100%\"><tr><td align=\"center\">\n";
#    print STDOUT "<table border=\"1\"><tr><td>\n";
#    print STDOUT "<table border=\"0\"><tr>\n";
#    print STDOUT "<td align=\"center\" valign=\"middle\"><img src=\"$iprs\" border=\"0\"></td>\n";
#    print STDOUT "<td align=\"center\" valign=\"middle\"><img src=\"$toolbox\" border=\"0\"></td>\n";
#    print STDOUT "</tr>\n";
#    print STDOUT "<tr><form action=\"$wget\" method=\"POST\" enctype=\"multipart/form-data\">\n";
#    print STDOUT "<tr><td align=\"center\" valign=\"middle\" colspan=\"2\"><b>Paste your proteic sequence below to calculate its crc64</b></td></tr>\n";
#    print STDOUT "<td align=\"center\" valign=\"middle\" colspan=\"2\"><input type=\"hidden\" name=\"toolbox\" value=\"1\">\n";
#    print STDOUT "<textarea cols=\"60\" rows=\"10\" name=\"sequence\" value=\"\"></textarea>\n";
#    print STDOUT "</td></tr>\n";
#    print STDOUT "<tr>\n";
#    print STDOUT "<td align=\"left\"><b>Upload a sequence file&nbsp;</b><input type=\"file\" name=\"upfile\"></td>\n";
#    print STDOUT "<td align=\"right\"><input type=\"hidden\" name=\"getcrc\" value=\"1\"><input type=\"submit\" value=\"Check CRC\"></td>\n";
#    print STDOUT "</form></tr>\n";
#    print STDOUT "</table></td></tr></td></tr></table></td></tr></table>\n";
#    print STDOUT "</body></html>\n";
#
#    return(1, '');
#
#}
#
#=head2 searchforCRC
#    
#  Description: Calculate the crc64 of a sequence or a sequence file.
#  Arguments:   $tool reference to a Dispatcher::Tool::InterProScan object.  
#  Returns:     1, '' on success
#               0, message on error
#
#=cut
#
#sub searchforCRC{
#
#    my($tool, $arnum, $arinfo) = @_;
#
#    return(0, "no tool reference given") unless($tool && $tool->isa("Dispatcher::Tool::InterProScan"));
#
#    my ($crc64, $length, $id, $nseq) = ('', '', '', 0);
#    my $icon = $tool->getConfigValue('imgurl.icon');
#    my $wget = $tool->getConfigValue('wgeturl.index');
#    my ($fh, $seq) = ('', '');
#    my $crc = { };
#    my $params = ['sequence', 'upfile'];
#
#    if($arnum && $arinfo){
#      for(@$arinfo){
#	($nseq, $id, $length, $crc64) = split(/\./, $_);
#	$crc->{$nseq}->{id} = $id;
#	$crc->{$nseq}->{len} = $length;
#	$crc->{$nseq}->{crc} = $crc64;
#      }
#    }
#
#    unless(keys %$crc){
#      for(@$params){
#
#	my $fh = $query->upload($_);
#
#	if($fh){
#	  while(<$fh>){
#	    next if($_ =~ /^\s*$/);
#	    chomp;
#	    if($_ =~ /^\s*>([^\s\,\;]+)/){
#	      if($seq){
#		$crc->{$nseq}->{len} = length($seq);
#		$crc->{$nseq}->{crc} = $tool->crc64($seq);
#	      }
#	      $nseq++;
#	      $crc->{$nseq}->{id} = $1;
#	      $seq = '';
#	    }else{
#	      $seq .= $_;
#	    }
#	  }
#	  $crc->{$nseq}->{len} = length($seq);
#	  $crc->{$nseq}->{crc} = $tool->crc64($seq);
#
#	}elsif($seq = $query->param($_)){
#	  my @seqline = ();
#	  @seqline = split(/\n/, $seq);
#	  for(@seqline){
#	    print STDERR "$_";
#	  }
#	  print STDERR "\n\n\n";
#	  my $sequence = '';
#	  my $cnt;
#
#	  foreach my $line (@seqline){
#	    next if($line =~ /^\s*$/);
#	    $line =~ s/\n//g;
#	    print STDERR "LINE=$line\n";
#	    if($line =~ /^\s*>([^\s\,\;]+)/){
#	      if($sequence){
#		$crc->{$nseq}->{len} = length($sequence);
#		$crc->{$nseq}->{crc} = $tool->crc64($sequence);
#	      }
#	      $nseq++;
#	      $crc->{$nseq}->{id} = $1;
#	      $sequence = '';
#	    }
#	    else{
#	      $cnt++;
#	      $sequence .= $line;
#
#	    }
#	    print STDERR "ok $cnt $line\nok $cnt $sequence\n";
#	  } #end foreach
#	  $crc->{$nseq}->{len} = length($sequence);
#	  $crc->{$nseq}->{crc} = $tool->crc64($sequence);
#	}
#      } #end for params
#    }
#    
#    print STDOUT header();
#    print STDOUT "<head>\n<link rel=\"SHORTCUT ICON\" href=\"$icon\">\n";
#    print STDOUT "<title>InterProScan Mini ToolBox - CRC64 results</title></head>\n";
#    print STDOUT "<body>\n";
#    print STDOUT "<table border=\"0\">\n";
#    print STDOUT "<tr><form action=\"$wget\" method=\"GET\"><td align=\"left\"><input type=\"hidden\" name=\"toolbox\" value=\"1\"><input type=\"submit\" value=\"Back To ToolBox\"></td></form></tr></table><br>\n";
#    print STDOUT "<table border=\"0\">\n";
#    print STDOUT "<form action=\"$wget\" method=\"POST\" enctype=\"multipart/form-data\">\n";
#    print STDOUT "<input type=\"hidden\" name=\"toolbox\" value=\"1\"><input type=\"hidden\" name=\"viewEntry\" value=\"1\">\n";
#    print STDOUT "<tr bgcolor=\"#cccccc\"><td>Press the button to see IprMatches entries for selected sequence(s)&nbsp;</td><td align=\"right\"><input type=\"submit\" value=\"Show\"><td></tr>\n";
#    print STDOUT "<tr><td colspan=\"2\"><table border=\"0\" cellspacing=\"0\" cellpadding=\"3\">\n";
#    
#    if(keys %$crc){
#      for(sort {$a <=> $b} keys %$crc){
#	my $num = $_;
#	my $id  = $crc->{$_}->{id};
#	my $c64 = $crc->{$_}->{crc};
#	my $len = $crc->{$_}->{len};
#	print STDOUT "<tr bgcolor=\"#dddddd\"><td align=\"left\"><input type=\"checkbox\" name=\"seqn\" value=\"$_\">\n";
#	print STDOUT "<input type=\"hidden\" name=\"info\" value=\"$num.$id.$len.$c64\">\n";
#	print STDOUT "<td align=\"left\"><b>Sequence&nbsp;$_&nbsp;:</b>&nbsp;$id</td>\n";
#	print STDOUT "<td align=\"left\"><b>CRC64&nbsp;:</b>&nbsp;$c64</td>\n";
#	print STDOUT "<td align=\"left\"><b>Length&nbsp;:</b>&nbsp;$len</td></tr>\n";
#
#	if($arnum && grep { /^$num$/ } @$arnum){
#	  my($res, $inx, $netry);
#	  ($res, $inx) = Index::IprMatches->new(undef, 1);
#	  return(0, $inx) unless $res;
#	  ($res, $entry) = $inx->getEntry($c64);
#         $entry = $entry->[0];   #getEntry retruns an array ref
#	  $entry = xml_quote($entry);
#	  $entry = getLinkedInfo($entry);
#	  print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"left\" colspan=\"4\">IprMatches entry for $crc->{$num}->{id}:<br><pre>$entry</pre></td></tr>\n" if($entry);
#	  print STDOUT "<tr bgcolor=\"#eeeeee\"><td align=\"left\" colspan=\"4\">Your entry $crc->{$num}->{id} returned nothing</td></tr>\n" unless($entry);
#	}
#	
#      }
#    }else{
#      return(0, "Could not find any sequences");
#    }
#    print STDOUT "</table></td></tr>\n";
#    print STDOUT "<tr bgcolor=\"#cccccc\"><td>Press the button to see IprMatches entries for selected sequence(s)&nbsp;</td><td align=\"right\"><input type=\"submit\" value=\"Show\"><td></tr></form></table>\n";
#    print STDOUT "<table border=\"0\">\n";
#    print STDOUT "<tr><form action=\"$wget\"><td align=\"left\"><input type=\"hidden\" name=\"toolbox\" value=\"1\"><br><input type=\"submit\" value=\"Back To ToolBox\"></td></form></tr>\n";
#    print STDOUT "</form></table>\n";
#    print STDOUT "</body></html>\n";

#    return(1, '');
#}
